/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.dialogs.AboutDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;

/**
 * Creates a menu bar for a specific module (dynamic)
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class DCMenuBar extends JMenuBar {

    private final DCModule module;
    private final AboutDialog aboutDialog = new AboutDialog();

    /**
     * Initializes this menubar for the specified module
     * @param module
     */
    public DCMenuBar(DCModule module) {
        this.module = module;
    	build();
    }
    
    /**
     * Opens the record form for a new Object
     */
    private void openRecordFormNew() {
        DCObject dco = module.getDataCrowObject();
        new RecordForm(false, 
                       false, 
                       dco, 
                       module.isImdbUpdateAllowed(),
                       module.isAmazonUpdateAllowed(), 
                       module.getInsertPanel().getClass(), 
                       dco.getUIRowID());
    }
    
    /**
     * Opens the record form for an existing Object
     */
    private void openRecordFormExist() {
        int tab = DataCrow.mainFrame.getSelectedTab();
        if (tab == MainFrame._INSERTTAB) {
            module.getInsertPanel().openRecord();
        } else {
            module.getSearchPanel().openRecord();
        }
    }    
    
    /**
     * Deletes the selected rows (from the database)
     */
    private void deleteRecords() {
    	module.getSearchPanel().deleteValues();
    }
    
    /**
     * Changes the current active module
     * @param moduleIndex newly selected module 
     */
    private void changeModule(int moduleIndex) {
        int activeModule = Settings.getValueAsInt(Repository.settings.stModule);
        
        if (moduleIndex != activeModule) {
        	DataCrow.mainFrame.changeModule(moduleIndex);
        }
    }
    
    /**
     * Builds this menubar
     */
    private void build() {
        String dcoName = module.getObjectName();
        
        JMenu menuTools = module.getToolsMenu();
        JMenu menuSettings = ComponentFactory.getMenu(Resources.getText(DCLabels.lblSettings));
        JMenu menuFile = ComponentFactory.getMenu(Resources.getText(DCLabels.lblFile));
        JMenu menuInformation = ComponentFactory.getMenu(Resources.getText(DCLabels.lblInformation));

        // information ment
        JMenuItem menuItemAbout = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblAbout));
        menuItemAbout.addActionListener(new AboutMenuAction());
        menuInformation.add(menuItemAbout);
        
        // file menu
        JMenuItem menuItemLog = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblLogEvent));
        menuItemLog.addActionListener(new EventViewerMenuAction());
        
        JMenuItem menuItemNew = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblNewItem, dcoName));
        menuItemNew.addActionListener(new NewMenuAction());
        JMenuItem menuItemOpen = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblOpenItem, dcoName));
        menuItemOpen.addActionListener(new OpenMenuAction());
        JMenuItem menuItemDelete = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblDeleteItem, dcoName));
        menuItemDelete.addActionListener(new DeleteMenuAction());
        JMenuItem menuItemQuit = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblQuit));
        menuItemQuit.addActionListener(new QuitMenuAction());
        
        menuFile.add(menuItemLog);
        menuFile.addSeparator();
        menuFile.add(menuItemNew);
        menuFile.add(menuItemOpen);
        menuFile.add(menuItemDelete);
        menuFile.addSeparator();
        menuFile.add(menuItemQuit);

        // module submenu
        JMenu subMenuModule = ComponentFactory.getMenu(Resources.getText(DCLabels.lblActiveModule));
        JMenuItem menuSoftware = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblModuleSoftware));
        menuSoftware.addActionListener(new SoftwareModuleMenuAction());
        JMenuItem menuMusicFile = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblModuleMusicFile));
        menuMusicFile.addActionListener(new MusicFileModuleMenuAction());
        JMenuItem menuMovie = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblModuleMovie));
        menuMovie.addActionListener(new MovieModuleMenuAction());
        JMenuItem menuAudioCD = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblModuleAudioCD));
        menuAudioCD.addActionListener(new AudioCDModuleMenuAction());
        subMenuModule.add(menuSoftware);
        subMenuModule.add(menuMovie);
        subMenuModule.add(menuMusicFile);
        subMenuModule.add(menuAudioCD);
        
        // settings menu
        JMenuItem menuItemSettings = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblSettings));
        menuItemSettings.addActionListener(new SettingsMenuAction());
        
        JMenuItem menuItemShowModList = ComponentFactory.getMenuItem(Resources.getText(DCLabels.lblToggleModuleList));
        menuItemShowModList.addActionListener(new ShowModuleListAction());
        
        menuSettings.add(menuItemSettings);
        menuSettings.add(menuItemShowModList);
        menuSettings.add(subMenuModule);
        
        this.add(menuFile);
        this.add(menuSettings);
        
        if (menuTools != null) {
        	this.add(menuTools);
        }
        
        this.add(menuInformation);
    }
    
    private class EventViewerMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            MessagesWriter._windowShow = true;
            MessagesWriter._windowToFront = true;
        }
    }    
    
    private class ShowModuleListAction implements ActionListener {
    	public void actionPerformed(ActionEvent arg0) {
            DataCrow.mainFrame.toggleModuleList();
    	}
    }

    private class SoftwareModuleMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            changeModule(DCModules._SOFTWARE);
        }
    }

    private class MovieModuleMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            changeModule(DCModules._MOVIE);
        }
    }

    private class MusicFileModuleMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            changeModule(DCModules._MUSIC);
        }
    }

    private class AudioCDModuleMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            changeModule(DCModules._AUDIOCD);
        }
    }
    
    private class SettingsMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DataCrow.mainFrame.showSettingsView();
        }
    }

    private class DeleteMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        	deleteRecords();
        }
    }
    
    private class QuitMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DataCrow.mainFrame.finish();
        }
    }
    
    private class OpenMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            openRecordFormExist();
        }
    }     

    private class AboutMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            aboutDialog.setVisible(true);
        }
    } 
    
    private class NewMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            openRecordFormNew();
        }
    }        
}
