/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.DCTable;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.Requestor;
import net.sf.dc.core.objects.RequestorCollection;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Base panel for all internet search forms
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public abstract class InternetSearchForm extends DCFrame {
    
    protected final WorkFlow wf = new WorkFlow();
    
    private final Vector vData = new Vector();
    
    private boolean overwrite = Settings.getValueAsBoolean(
    		                    Repository.settings.stODBOverwriteValues);
    private boolean broadband = Settings.getValueAsBoolean(
                                Repository.settings.stBroadband);
    
    protected JCheckBox checkOverwrite;
    protected JCheckBox checkBroadband;
    protected JTextField textServer = ComponentFactory.getTextField();
    protected JLabel labelServer = ComponentFactory.getLabel(
                Resources.getText(DCLabels.lblInternetServer));
    
    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();
    private JPanel panelSettings = new JPanel(false);
    
    protected JButton buttonStopSearch;
    protected final DCTable table;
    
    protected final Color color = 
        Settings.getValueAsColor(Repository.settings.stColor);
    
    private final RecordForm recordForm;
    protected final DCObject dco;
    protected final Class clazz;
    private final int rowNumber;
    
    protected InternetUpdateProcess process;
    
    private boolean cancelled = false;
    
    /**
     * Initializes this form for a specific data crow object. 
     * The results will be passed back on to the record form
     * @param dco
     * @param recordForm
     */
    public InternetSearchForm(DCObject dco, RecordForm recordForm) {
        this.dco = dco;
        this.recordForm = recordForm;
        this.clazz = null;
        this.rowNumber = -1;
        table = ComponentFactory.getDCTable(dco.getModule(), true);
        initialize();
    }
    
    /**
     * Initializes this form for a specific data crow object. 
     * The results will be passed back on to the specified class.
     * The rownumber indicates which row in table of the class should
     * be updated
     * @param dco 
     * @param clazz target class
     * @param rowNumber UI table row number
     */
    public InternetSearchForm(DCObject dco, Class clazz, int rowNumber) {
        this.dco = dco;
        this.clazz = clazz;
        this.recordForm = null;
        this.rowNumber = rowNumber;
        table = ComponentFactory.getDCTable(dco.getModule(), true);
        initialize();
    }
    
    public JPanel getSettingsPanel() {
    	return panelSettings;
    }
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }  
    
    /**
     * Adds an object to the table
     * @param dco
     */
    public void addObject(DCObject dco) {
        vData.addElement(dco);
    	table.addRow(dco.getValues());
    }
    
    /**
     * Adds a log message
     * @param message message to be added
     */
    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }    
    
    /**
     * Returns the selected data (in the table)
     */
    public DCObject getSelectedData() {
        int row = table.getSelectedRow();
        if (row < 0) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowForTransfer),
                           MessageBox._WARNING);
            return null;
        } 
        DCObject dco = (DCObject) vData.elementAt(row);
        return dco;
    }   
    
    /**
     * Opens a record form for the selected record
     */
    private void openRecord() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowToOpen),
                           MessageBox._WARNING);
            return;
        }
        
        DCObject dco = getSelectedData();
        if (dco != null) {
        	new RecordForm(true, false, dco, false, false, getClass(), -1);
        }
    }  
    
    /**
     * Cancels the internet search
     */
    private void stopSearch() {
        cancelled = true;
        if (process != null) {
        	process.stopSearch();
        }
        addMessage(Resources.getText(DCMessages.msgStoppedSearch));
        setSearchMode();
    }
    
    /**
     * Closes this form and applies all settings
     */
    protected void close() {
        Settings.setValue(Repository.settings.stODBOverwriteValues,
                          new Boolean(checkOverwrite.isSelected()));
        Settings.setValue(Repository.settings.stBroadband,
                          new Boolean(checkBroadband.isSelected()));
        Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                          getSize());     
        saveSettings();
        
    	stopSearch();
        dispose();
        setVisible(false);
    }
    
    /**
     * Transfers the selected result to the requestor 
     */
    public void transferRow() {
        DCObject dco = getSelectedData();
        
        if (dco == null) {
            return;
        }

        Object[] row = dco.getValues();
        String sMethod = "updateRowAt";
        
        Object[] params;
        RequestorCollection requestors = new RequestorCollection();
        if (recordForm != null) {
            sMethod = "setData";
            params = new Object[] {recordForm, row, new Boolean(overwrite)};
            Requestor request = new Requestor(recordForm.getClass(), sMethod, params);
            requestors.addRequestor(request);
        } else {
            params = new Object[] {new Integer(rowNumber), row, new Boolean(overwrite)};
            Requestor request = new Requestor(clazz, sMethod, params);
            requestors.addRequestor(request);
        }
        
        wf.displayResults(new LinkedList(), requestors, true);
        
        close();
    }   
    
    /**
     * Clears the content of the UI table
     */
    private void clearTable() {
    	table.setRowCount(0);
        vData.clear();
        updateProgressBar(0);
        textLog.setText("");
    }
    
    private void initialize() {
        buildDialog();
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        setIconImage(IconLibrary._icoInternet.getImage());
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);

        setDefaultValues();
        setSearchMode();        
        setVisible(true);
    }
    
    public abstract void setSearchMode();
    public abstract void setCancelMode();
    protected abstract void saveSettings();
    protected abstract void setDefaultValues();
    protected abstract JPanel getSearchPanel();    
    
    private void buildDialog() {
        setResizable(true);
        getContentPane().setLayout(Layout.getGBL());
        getContentPane().setBackground(color);

        //**********************************************************
        //Settings
        //**********************************************************
        JPanel panelSettings = new JPanel(false);
        panelSettings.setLayout(Layout.getGBL());
        panelSettings.setBackground(color);
        
        checkOverwrite = ComponentFactory.getCheckBox(
                         Resources.getText(DCLabels.lblOverwriteExistingValues));
        checkOverwrite.setBackground(color);
        checkOverwrite.setToolTipText(Resources.getText(DCTooltips.tpOverwriteExistingValues));
        checkOverwrite.setSelected(overwrite);

        checkBroadband = ComponentFactory.getCheckBox(
                         Resources.getText(DCLabels.lblBroadbandConnection));
        checkBroadband.setBackground(color);
        checkBroadband.setSelected(broadband);
        
        textServer.setPreferredSize(new Dimension(200, 23));
        
        panelSettings.add(checkOverwrite, Layout.getGBC( 0, 0, 10, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						  new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(checkBroadband, Layout.getGBC( 0, 1, 10, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						  new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(labelServer, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						  new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(textServer, Layout.getGBC(  1, 2, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
						  new Insets(5, 5, 5, 5), 0, 0));
        
        panelSettings.setBorder(ComponentFactory.getTitleBorder(Resources.getText(DCLabels.lblSettings)));        
        
        //**********************************************************
        //Search panel
        //**********************************************************
        JPanel panelSearch = getSearchPanel();
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel(false);
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.setBackground(color);
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0)); 
        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel(false);
        panelLog.setLayout(Layout.getGBL());
        panelLog.setBackground(color);
        
        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));
        
        panelLog.setBorder(ComponentFactory.getTitleBorder(
        	               Resources.getText(DCLabels.lblLog)));
        
        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResults = new JPanel(false);
        panelResults.setLayout(Layout.getGBL());
        panelResults.setBackground(color);
            
        JScrollPane scrollTable = new JScrollPane(table);
        table.addMouseListener(new TableMouseListener());
        scrollTable.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        
        table.removeHiddenFields(dco.getHiddenFields());
        
        panelResults.add(scrollTable,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));
         
        //**********************************************************
        //Actions panel
        //**********************************************************        
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);  
        
        JButton buttonTransfer = ComponentFactory.getButton(
                                 Resources.getText(DCLabels.lblTransfer));
        buttonStopSearch =  ComponentFactory.getButton(
                            Resources.getText(DCLabels.lblStop));
        buttonStopSearch.addActionListener(new StopSearchAction());

        JButton buttonClear = ComponentFactory.getButton(
                              Resources.getText(DCLabels.lblClear));
        buttonClear.addActionListener(new ClearAction());
        
        JButton buttonClose = ComponentFactory.getButton(
                                 Resources.getText(DCLabels.lblClose));
        buttonClose.setToolTipText(Resources.getText(DCTooltips.tpClose));
        buttonTransfer.setToolTipText(Resources.getText(DCTooltips.tpTransfer));
        
        buttonTransfer.addActionListener(new TransferRowAction());
        buttonClose.addActionListener(new CancelAction());
        
        panelActions.add(buttonTransfer,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonStopSearch,Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClear,     Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						new Insets(5, 5, 5, 5), 0, 0));        
        panelActions.add(buttonClose,     Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Main Panel
        //**********************************************************
        if (panelSearch != null) {
            getContentPane().add(panelSearch,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets(5, 5, 5, 5), 0, 0));
        }
        getContentPane().add(panelSettings, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));  
        getContentPane().add(panelResults,  Layout.getGBC( 0, 2, 1, 1, 50.0, 50.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
							 new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelProgress, Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelLog,      Layout.getGBC( 0, 5, 1, 1, 10.0, 10.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));        
        pack();
    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
    
    private class StopSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            stopSearch();
        }
    }
    
    private class ClearAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearTable();
        }
    }    
    
    private class TransferRowAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            transferRow();   
        }
    }    

    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                openRecord();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
