/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JTabbedPane;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.note.NotePanel;
import net.sf.dc.console.sql.SQLPanel;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.Requestor;
import net.sf.dc.core.objects.RequestorCollection;
import net.sf.dc.db.Database;
import net.sf.dc.db.DatabaseConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.jflexsett.settings.SettingsFile;
import net.sf.ext.jflexsett.settings.ui.SettingsView;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Holder of all the UI panels. The panels are shown as tabs.
 * By using this frame background colors can be set and requests can be past
 * to the panels by using the @see RequestorCollection class.
 * At the top of the frame a menubar is shown. From this menu the module can
 * be selected, the log form and record form can be opened and the MP3
 * organizer can be selected.
 * 
 * NOTE: I have specifically chosen to load all the panels at once. 
 *       Of course one can say this is heavy on the resources, but.. 
 *       There are just a couple of panels. Switching between modules
 *       should be smooth. Constantly needing to rebuild panels and 
 *       other UI stuff would make the application slow
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class MainFrame extends JFrame {
    
    public static final int _SEARCHTAB = 0;
    public static final int _INSERTTAB = 1;
    
    private final JMenuBar menuBar = new JMenuBar();
    
    private final ModuleListPanel moduleListPanel = new ModuleListPanel();
    private final JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();
    private final SQLPanel sqlPanel = new SQLPanel();
    private final NotePanel notePanel = new NotePanel();

    private SettingsView settingsView;
    private DCModule activeModule = null;

    /**
     * Initializes all the panels and puts them in a tabbed pane. The menubar
     * is build and its actions specified. The background color for all UI
     * components is set
     */
    public MainFrame() {
        super();
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE); 
        
        this.getContentPane().setLayout(Layout.getGBL());
        
        // Initializes the settings editor
        createSettingsView();

        toggleModuleList(Settings.getValueAsBoolean(Repository.settings.stShowModuleList));
        
        try {
            setTitle(Resources.getText(DCLabels.lblMainTitle));
            setResizable(false);
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    finish();
                }
            });

            Image image = IconLibrary._icoMain.getImage();
            setIconImage(image);
            image.flush();

            int module = Settings.getValueAsInt(Repository.settings.stModule);
            changeModule(module);
            
            getContentPane().add(moduleListPanel, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.VERTICAL,
                                 new Insets(0, 0, 0, 0), 0, 0));
            getContentPane().add(tabbedPane, Layout.getGBC( 1, 0, 1, 1, 20.0, 20.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(0, 0, 0, 0), 0, 0));
            
            tabbedPane.setPreferredSize(Settings.getValueAsDimension(Repository.settings.stTabbedPaneSize));
            pack();
            tabbedPane.setSize(Settings.getValueAsDimension(Repository.settings.stTabbedPaneSize));
            
            setHiddenFields();
        } catch (Error err) {
            Messages.setMessage(err);
        }
        this.setBackgroundColor(
                Settings.getValueAsColor(Repository.settings.stColor));
        
        this.setRequiredFields();
        this.setResizable(true);
        this.setSize(Settings.getValueAsDimension(Repository.settings.stMainViewSize));
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
    }
    
    public void setSelectedTab(int index) {
    	tabbedPane.setSelectedIndex(index);
    }
    
    public int getSelectedTab() {
        return tabbedPane.getSelectedIndex();
    }
    
    public void showSettingsView() {
    	settingsView.setVisible(true);
    }
    
    private void toggleModuleList(boolean enable) {
        moduleListPanel.setVisible(enable);
    }
    
    public void toggleModuleList() {
    	moduleListPanel.setVisible(!moduleListPanel.isVisible());
        Settings.setValue(Repository.settings.stShowModuleList, 
                          new Boolean(moduleListPanel.isVisible()));
        
    }
    
    /**
     * Applies the required fields for all modules. A required
     * field will mostly be shown as a red header in the table.
     */
    public void setRequiredFields() {
        for (Enumeration enum = DCModules.getModules(); enum.hasMoreElements(); ) {
            DCModule module = (DCModule) enum.nextElement();
            if (module.getSearchPanel() != null) {
                module.getSearchPanel().setRequiredFields();
            }
            
            if (module.getInsertPanel() != null) {
            	module.getInsertPanel().setRequiredFields();
            }
        }
        this.repaint();
    }
    
    /**
     * Removes all the fields from the table which are not selected as 
     * visible fields. These fields are solely removed from the table,
     * not from the tablemodel. They will be updated and used in validations.
     */
    public static void setHiddenFields() {
        for (Enumeration enum = DCModules.getModules(); enum.hasMoreElements(); ) {
            DCModule module = (DCModule) enum.nextElement();
            int[] hiddenFields = Settings.getValueAsIntegerArray(
                                    module.getHiddenFieldsSettingsKey());
            
            DCFieldDefinition[] fields = module.getFieldDefinitions();
            for (int j = 0; j < fields.length; j++) {
                fields[j].setHidden(false);
            }
            
            for (int i = 0; i < hiddenFields.length; i++) {
                fields[hiddenFields[i]].setHidden(true); 
            }
            
            if (module.getSearchPanel() != null) {
                module.getSearchPanel().removeHiddenFields(hiddenFields);
            }
            
            if (module.getInsertPanel() != null) {
                module.getInsertPanel().removeHiddenFields(hiddenFields);
            }
        }
    }
    
    /**
     * Applies the background color to all panels and their components
     * @color   background color
     */
    public void setBackgroundColor(Color color) {
        for (Enumeration enum = DCModules.getModules(); enum.hasMoreElements(); ) {
            DCModule module = (DCModule) enum.nextElement();
            
            if (module.getSearchPanel() != null) {
                module.getSearchPanel().setBackground(color);
            }   
            
            if (module.getInsertPanel() != null) {
            	module.getInsertPanel().setBackground(color);
            }
        }            
  
        settingsView.setBackgroundColor(color);            
        sqlPanel.setBackground(color);
        notePanel.setBackground(color);
    }

    /**
     * Sets the module. Only the tabs for this module are shown. The module
     * is selected from the module combobox from the menubar
     */
    public void changeModule(int index) {
        activeModule = DCModules.getModule(index);
        
        Settings.setValue(Repository.settings.stModule, new Integer(index));
        
        moduleListPanel.setSelectedIndex(index);
        setJMenuBar(null);
        setJMenuBar(activeModule.getMenuBar());
        
        Dimension size = tabbedPane.getSize();
        
        tabbedPane.setPreferredSize(size);
        tabbedPane.removeAll();
        
        DataCrowPanel[] panels = activeModule.getPanels();
        for (int i = 0; i < panels.length; i++) {
            tabbedPane.addTab(panels[i].getTitle(), 
                              panels[i].getIcon(), 
							  panels[i]);
        }
        
        tabbedPane.addTab(Resources.getText(DCLabels.lblTabSQL), 
                          IconLibrary._icoSqlManager, 
                          sqlPanel);
        tabbedPane.addTab(Resources.getText(DCLabels.lblTabNote), 
                          IconLibrary._icoNote, 
                          notePanel);
        
        tabbedPane.setSize(size);
        tabbedPane.setPreferredSize(size);
    }
    
    /**
     * Closes the application. If running in client <> server mode, a question 
     * will popup to close the server as well.
     */
    public void finish() {
        Messages.setMessage(
            Resources.getText(DCMessages.msgApplicationStops),
            Messages._STANDARD);

        if (DatabaseConnection.bServerConnection && 
            Settings.getValueAsBoolean(Repository.settings.stCloseServerQuestion)) {
        
            new QuestionBox(Resources.getText(DCMessages.msgCloseServer));
                               
            if (QuestionBox.result.booleanValue()) {
                WorkFlow wf = new WorkFlow();
                wf.runQueryDirect("SHUTDOWN");
            }
        } else {
            Database.close();
        }

        Settings.setValue(Repository.settings.stTabbedPaneSize, tabbedPane.getSize());
        Settings.setValue(Repository.settings.stMainViewSize, getSize());
        SettingsFile.save();
        clearCache();
        System.exit(0);
    }
    /**
     * Passes requests to the correct module (read, the UI component of
     * the module). 
     * 
     * @param requestors a collection of requests (actions) to be passed 
     *                   to the target
     * @param data data which should be passed to the target. Can be null
     * @param qryWasSuccess Indicates if the query was executed succesfully.
     *                      Unsuccessfull queries can still result in the
     *                      actions being executed (if specified)
     */
    public void displayResults( RequestorCollection requestors, 
                                LinkedList data, 
                                boolean qryWasSuccess) {
        Requestor[] reqObjects = requestors.toRequestorArray();

        try {
            for (int i = 0; i < reqObjects.length; i++) {
                Class clazz = reqObjects[i].getClazz();
                String sMethod = reqObjects[i].getMethod();
                Object[] parameters = reqObjects[i].getParameters();
    
                if (qryWasSuccess || (!qryWasSuccess && reqObjects[i].getExecuteOnFail())) {
                    // Specific actions to be performed by the record form
                    if (clazz.getName().equals(RecordForm.class.getName())) {
                        if (sMethod.equals("setData")) {
                            RecordForm form = (RecordForm) parameters[0];
                            Object[] row = (Object[]) parameters[1];
                            boolean overwrite = ((Boolean) parameters[2]).booleanValue();
                            form.setData(row, overwrite);
                            form = null;
                        } 
                    } else if (clazz.getName().equals(TablePanel.class.getName())) {
                    // Specific actions to be performed by the table panel
                        if (sMethod.equals("closeRecord") && parameters[0] != null) {
                            ((RecordForm) parameters[0]).close();
                            parameters[0] = null;
                        }
                    } else {
                    // Actions to be performed by one of the search / insert / note or
                    // sql panels of one of the modules.
                        if (parameters == null && data != null) {
                            parameters = new Object[1];
                            parameters[0] = data;
                        }
                        executeMethod(clazz, sMethod, parameters);
                    }    
                } 
            }
        } finally {
            if (data != null) {
            	data.clear();
            	data = null;
            }
        }
    } 
    
    /**
     * Method invocation. Executes the method on the class using the specified
     * parameters. 
     * 
     * @param clazz The class on which the action will be executed
     * @param sMethod The action to be executed
     * @param parameters The parameters for the to be executed action
     */
    private void executeMethod(Class clazz, String sMethod, Object[] parameters) {
        for (Enumeration enum = DCModules.getModules(); enum.hasMoreElements(); ) {
        	DCModule module = (DCModule) enum.nextElement();
             
            Object target = null;
            if (module.getSearchPanel() != null &&
            	clazz.getName().equals(module.getSearchPanel().getClass().getName())) {
                target = module.getSearchPanel();
            } else if (module.getInsertPanel() != null &&
                       clazz.getName().equals(module.getInsertPanel().getClass().getName())) {
                target = module.getInsertPanel();
            } else if (clazz.getName().equals(sqlPanel.getClass().getName())) {
                target = sqlPanel;
            } else if (clazz.getName().equals(notePanel.getClass().getName())) {
                target = notePanel;
            }
            
            if (target != null) {
                Class[] classes = null;
                if (parameters != null) {
                	classes = new Class[parameters.length];
                	for (int i = 0; i < classes.length; i++) {
                        if (parameters[i].getClass().equals(Boolean.class)) {
                        	classes[i] = boolean.class;
                        } else if (parameters[i].getClass().equals(Integer.class)){
                        	classes[i] = int.class;
                        } else {
                            classes[i] = parameters[i].getClass();
                        }
                	}
                }
                
                Method method = null;
                try {
                	method = clazz.getMethod(sMethod, classes);
                    method.invoke(target, parameters);
                } catch (Exception exp) {
                    Messages.setMessage("SEVERE SYSTEM ERROR! Could not execute " + sMethod + 
                                        " on class " + clazz.getName() + ". error: " + exp,
                                        Messages._ERROR);
                    new MessageBox("SEVERE SYSTEM ERROR! Could not execute " + sMethod + 
                                   " on class " + clazz.getName() + ". error: " + exp,
                                   MessageBox._ERROR);
                } finally {
                    if (method == null) {
                        Messages.setMessage("SEVERE SYSTEM ERROR! Could not execute " + sMethod + " on class " + clazz.getName(),
                        		            Messages._ERROR);
                    }
                    method = null;
                	classes = null;
                    parameters = null;
                }
            }
        }
    }
    
    /**
     * Removes the cache from the cache directory. The cache directory
     * itself will always remain after its creation.
     * Cache consists mostly out of pictures retrieved from the database
     */
    private void clearCache() {
        try {
            String sCacheDir = Utilities.getCacheDirectory();
            File file = new File(sCacheDir);
            if (file.exists() && file.isDirectory()) {
                String[] files  = file.list();
                
                if (files != null) {
                    for (int i = 0; i < files.length; i++) {
                        File cacheFl = new File (sCacheDir + files[i]);
                        cacheFl.delete();
                    }
                }
            }
        } catch (Exception exp) {
            Messages.setMessage(Resources.getText(DCMessages.msgCouldNotClearCache),
                                Messages._ERROR);
            Messages.setMessage(exp);
        }
    }
    
    /**
     * Initializes the Settings Vies. JFlexSett is used to create the
     * settings panel. 
     */
    private void createSettingsView() {
        settingsView = new SettingsView("Data Crow Settings Panel");
        settingsView.setDisclaimer(
            "              __                       " + '\n' +
            "       <-----/@@\\----->               " + '\n' +
            "      <-< <  \\\\//  > >->             " + '\n' +
            "        <-<-\\ __ /->->                " + '\n' +
            "        Data /  \\ Crow                " + '\n' +
            "            ^    ^                     " + '\n' +
            "Created By Robert-Jan van der Waals" + '\n' +
            "Original product: CD Library 3.03" + '\n' +
            "" + '\n' +
            "This product includes Hypersonic SQL."  + '\n' +
            "Originally developed by Thomas Mueller and the Hypersonic SQL Group"  + '\n' +
            ""  + '\n' +
            "This product used JFlexSett 0.1 alpha" + '\n' +
            "Created By Robert-Jan van der Waals" + '\n' +
            "" + '\n' +
            "Find me at http://sourceforge.net/projects/datacrow");
        settingsView.setSize(Settings.getValueAsDimension(Repository.settings.stSettingsViewSize));
        Image image = IconLibrary._icoSettings.getImage();
        settingsView.setIconImage(image);
        image.flush();
    }    
}