/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.da.ExportToPdfDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.Requestor;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

/**
 * Base for all insert panels (for objects like Software, Music and Movie)
 * Has an implemenation for supporting a child panel (in case of an Object
 * with children). The functionality is not fully implemented in this class
 * as only (for now) the Audio Track and Audio CD panel have a parent-child
 * relation. 
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public abstract class TableInsertPanel extends TablePanel {
    
    protected String parentID = null;
    protected TableInsertPanel childPanel;

    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created. 
     * @param module the module for which this panel is created
     * @param title the title of this panel
     * @param icon the icon of this panel
     */
    public TableInsertPanel (DCModule module, String title, ImageIcon icon) {
        super(module, title, icon);        
        
        boolean imdb = getModule().isImdbUpdateAllowed();
        boolean amazon = getModule().isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCategoryForm.CatagoryInsertPanel ?
                       false : true;
        
        JComponent[] extraComponents = getAllExtraComponents();
        
        panelTableActions = new TableActionsPanel(true, extraComponents);
        panelTableActions.addSaveActionListener(new SaveNewValuesAction());
            
        popupMenu = new PopupMenu(imdb, amazon, false, true, open, false);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        
        buildPanel();
    }
    
    /**
     * Adds a list of data crow objects to the table. Mostly this
     * is used by internet updates (retrieving information from another source). 
     * The data added is ready for saving. In case the object has children, 
     * these are added to this table as well.
     * @param data list of data crow objects
     * @param addUIRowID display the UI row ID
     */
    public void addRow(LinkedList data, boolean addUIRowID) {
        if (hasChild()) {
            // first clear the child panel of all shown records.
            // this prevents new child records to be added to the wrong
            // parent
        	childPanel.clearTable();
        }
        
        while (data.size() > 0) {
            DCObject lcObj = (DCObject) data.removeFirst();
            if (hasChild() && lcObj.getChildren().length > 0) {
                LinkedList children = new LinkedList();
                DCObject[] dcoChildren = lcObj.getChildren();
                for (int i = 0; i < dcoChildren.length; i++) {
                	children.add(dcoChildren[i]);
                }
                childPanel.addRow(children, true);
            }
            
            Object[] row = lcObj.getValues();
            if (addUIRowID && (row[0] == null || row[0].equals(""))) {
                row[0] = "" + (table.getRowCount() + 1);
                if (hasChild()) {
                    // add the child records to the case
                	childPanel.rememberChildValues(row[0].toString());
                }
            }
            
            table.addRow(row);
            lcObj = null;
            row = null;
        }
        
        data.clear();
        data = null;
    }
    
    /**
     * Opens the selected record
     */
    public void openRecord() {
        cancelEdit();
        
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowToOpen),
                           MessageBox._WARNING);
        } else {
        	DCObject dco = getObjectForSaving(selectedRow);
            dco.removeRequestors();
            openRecord(dco, false);
        }
    }    
    
    /**
     * Creates a PDF report. All visible fields are reported on.
     * A file browser is opened in which the user can indicate where
     * the pdf file should be stored.
     */
    public void createPdfReport() {
        DCObject[] objects = new DCObject[table.getRowCount()];
        for (int i = 0; i < objects.length; i++) {
            objects[i] = getObjectForSaving(i); 
        }
        
        ExportToPdfDialog dialog = new ExportToPdfDialog(objects);
        dialog.setVisible(true);
    } 
    
    /**
     * Set the parent ID (if this panel is acting as a child)
     * @param sID parent ID
     */
    public void setParentID(String sID) {
        parentID = sID;
    }
    
    /**
     * Clear this table and the child table
     */
    public void clearTable() {
        super.clearTable();
        
        if (hasChild()) {
            childPanel.clearTable(true);
        }
    } 
    
    public void clearTable(boolean cache) {}
    
    /**
     * Retrieve module specific components
     */
    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];
        if (hasChild()) {
            JComponent[] childComponents = getChildModule().getInsertPanel().getExtraComponents();
            JComponent[] parentComponents = getExtraComponents();
            
            components = new JComponent[parentComponents.length + childComponents.length];
            int counter = 0; 
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    public abstract JComponent[] getExtraComponents();    

    /**
     * Retrieves a data crow object for the specified row for saving purposes.
     * Children are added to this object
     * @param row row number
     */
    public DCObject getObjectForSaving(int row) {
        DCObject dco = table.getObjectForRow(row, true);
        dco.setBatch(true);
        dco.setSilence(true);
        dco.setUIRowNumber(row);

        dco.setUIRowNumber(row);
        String sID = table.getValueAt(row, 0, true).toString();
        
        dco.addRequestor(new Requestor(getClass(), "removeRows", 
                                       new Object[] {new int[] {row}}));
        if (hasChild()) {
            DCObject[] children = childPanel.getChildren(sID);
            
            for (int j = 0; j < children.length; j++) {
                dco.addChild(children[j]);
            }
        }
        return dco;
    }        
    
    /**
     * Remove the rows from the table with the given indices
     * @param rows the row indices 
     */
    public void removeRows(int[] rows) {
        try {
            table.setSelectedRow(0);
            
            if (hasChild()) {
                for (int i = 0; i < rows.length; i++) {
                    String sID = table.getValueAt(rows[i], 0, true).toString();
                    childPanel.removeChildren(sID);
                }
            }
            table.removeRows(rows);
        } catch (Exception ignore) {}
    }
    
    public void removeChildren(String parentID) {}

    /**
     * Saves the give rows to the database. Saving is done threaded and
     * can be cancelled by the user
     * @see SavingThread
     * @param rows row indices / to be saved rows
     */
    public void save(int[] rows) {
    	if (!isTaskRunning()) {
            task = new SavingThread(rows, getClass());
            task.start();
        } else {
            new MessageBox(Resources.getText(DCMessages.msgJobRunning),
                           MessageBox._WARNING);    
        }
    }
    
    /**
     * Saves all the new values (including children)
     */
    public void saveNewValues() {
        int rowCount = table.getRowCount();
        int[] rows = new int[rowCount];
        for (int i = 0; i < rowCount; i++) {
        	rows[i] = i;
        }
        save(rows);
    }
    
    /**
     * Saves the selected values only
     * @param silent indicates wether messages should be shown
     */
    public void saveSelectedValues(boolean silent) {
        cancelEdit();
        int[] rows = table.getSelectedRows();
        save(rows);
    }    
    
    public String getParentID() {
    	return parentID;
    }
    
    protected abstract void rememberChildValues(String id);
    protected abstract void loadChildValues(String id);
    protected abstract DCObject[] getChildren(String parentID);
    
    /**
     * Creates the panel
     */
    private void buildPanel() {
        JPanel panelInput  = new JPanel(false);
        JPanel panelStatus = new JPanel(false);

        
        //**********************************************************
        //Input panel
        //**********************************************************
        panelInput.setLayout(Layout.getGBL());
 
        table = ComponentFactory.getDCTable(getModule(), false);
        table.addMouseListener(tableMouseListener);

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panelInput.add( scrollResults,      Layout.getGBC( 0, 0, 7, 1, 20.0, 20.0
                       ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));
        
        if (hasChild()) {
            childPanel = getChildModule().getInsertPanel();
            
            table.getSelectionModel().addListSelectionListener(new RowSelectionListener());            
            
            JScrollPane scrollResults2 = new JScrollPane(getChildModule().getInsertPanel().table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            
            panelInput.add( scrollResults2,   Layout.getGBC( 0, 1, 7, 1, 20.0, 20.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }

        //**********************************************************
        //Status panel
        //**********************************************************
        panelStatus.setLayout(Layout.getGBL());
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        panelStatus.add(    labelStatus,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  0, 0, 0), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        panelInput.setBorder(ComponentFactory.getTitleBorder(
                             Resources.getText(DCLabels.lblInsert)));
        panelStatus.setBorder(ComponentFactory.getTitleBorder(
                              Resources.getText(DCLabels.lblStatus)));

        add(panelInput,     Layout.getGBC( 0, 0, 8, 1, 20.0, 20.0
                           ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                            new Insets( 5, 5, 5, 5), 0, 0));
        add(panelTableActions,Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets( 5, 5, 5, 5), 0, 0));
                            
        add(panelStatus,    Layout.getGBC( 0, 2, 8, 1, 1.0, 1.0
                           ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                            new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }
    
    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************

    /**
     * @author Nuclear Vision
     * @since 1.4
     * @version 1.5
     */
    private class SavingThread extends DataTask {
        public SavingThread(int[] rows, Class clazz) {
            super(rows, clazz);
        }
        
        public void run() {
            denyActions();
            startRunning();
            cancelEdit();
            
            if (hasChild()) {
                int row = table.getSelectedRow() != -1 ? table.getSelectedRow() : 0;
                childPanel.rememberChildValues(table.getValueAt(row, 0, true).toString());
            }
            
            if (rows.length == 0) {
                new MessageBox(Resources.getText(DCMessages.msgNoChangesToSave), 
                               MessageBox._INFORMATION);
                stopRunning();
            }             
            
            setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));

            boolean ignoreErrors = false;
            for (int i = rows.length - 1; i > -1 && keepOnRunning(); i--) {
                DCObject dco = getObjectForSaving(rows[i]);
                dco.setUIRowNumber(rows[i]);
                if (i == 0) {
                    dco.addRequestor(new Requestor(clazz, "setStatus", 
                                                   new String[] {Resources.getText(DCMessages.msgDataSaved)}));
                    dco.setEndOfBatch(true);
                }
                
                try {
                    dco.saveNew();
                } catch (ValidationException vExp) {
                    if (!ignoreErrors) {
                        SaveQuestionBox question = new SaveQuestionBox(vExp);
                        if (question.getResult() == SaveQuestionBox._CANCEL) {
                            stopRunning();
                        } else if (question.getResult() == SaveQuestionBox._IGNORE) {
                            ignoreErrors = true;
                        }
                    }
                }
                
                try {
                    sleep(300);
                } catch (Exception ignore) {} // doesn't want to sleep..                
            }
            allowActions();
            stopRunning();
        }
    }
    
    private class RowSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e){
            try {
                if (hasChild()) {
                    if (e.getValueIsAdjusting()) {
                        String idFrom = "";
                        String idTo = "";
                        if (e.getFirstIndex() == table.getSelectedRow()) {
                        	idFrom = "" + table.getValueAt( e.getLastIndex(), 0, true);
                            idTo = "" + table.getValueAt( e.getFirstIndex(), 0, true);
                        } else {
                            idFrom = "" + table.getValueAt( e.getFirstIndex(), 0, true);
                            idTo = "" + table.getValueAt( e.getLastIndex(), 0, true);
                        }
                        childPanel.rememberChildValues(idFrom);
                        childPanel.setParentID(idTo);
                        childPanel.loadChildValues(idTo);
                    } else if (table.getRowCount() == 1) {
                        String id = "" + table.getValueAt(0, 0, true);
                        childPanel.rememberChildValues(id);
                        childPanel.setParentID(id);
                    }
                } 
            } catch (Exception ignore) {}
        }
    }    
    
    /**
     * Saves all selected values 
     * @see TableInsertPanel#saveSelectedValues(boolean)
     */
    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            saveSelectedValues(false);
        }
    }  

    /**
     * Saves all new values (insert action)
     * @see TableInsertPanel#saveNewValues()
     */
    private class SaveNewValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNewValues();
        }
    }    
    
    /**
     * Starts the Amazon update job
     */
    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    /**
     * Starts the IMDB update job
     */
    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }      
}
