/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;

/**
 * Creates a panel for creating the search condition. The search condition exists 
 * out of fields with a value. On this combination will be searched... 
 * Hidden fields are not shown in this panel. Also support searching parent records
 * based on a condition on a child object / record 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.5
 * @version 1.50
 */
public class TableSearchConditionsPanel extends JPanel {

    private final Vector vInput = new Vector(0);
    private final Vector vLabels = new Vector(0);
    private final DCModule[] modules;
    private int[] module;
    
    /**
     * Initializes this panel and set the correct fields for searching.
     * Hidden fields are not shown as well notsearchable fields.
     * In case of a child module, the fields of both the parent and the child
     * are shown.
     * @param module the module for which this panel is created
     */
    public TableSearchConditionsPanel(DCModule module) {
        if (module != null && module.getChildModule() != null) {
             modules = new DCModule[2];
             modules[0] = module;
             modules[1] = module.getChildModule();
        } else if (module != null) {
            modules = new DCModule[1];
            modules[0] = module;
        } else {
            modules = new DCModule[0];
        }
        
        buildPanel(); 
        removeHiddenFields();
    }
    
    /**
     * Makes sure that hidden fields are not displayed
     */
    public void removeHiddenFields() {
        // first, set all fields to be visible
        for (int i = 0; i < vInput.size(); i++) {
            JComponent comp = (JComponent) vInput.elementAt(i);
            comp.setVisible(true);
            JLabel label = (JLabel) vLabels.elementAt(i);
            label.setVisible(true);                
        }
        
        int diff = 0;
        for (int i = 0; i < modules.length; i++) {
            DCObject dco = modules[i].getDataCrowObject();
            
            int[] hiddenFields = dco.getHiddenFields();
            int[] notSearchableFields = dco.getNotSearchableFields();
            
            for (int j = 0; j < hiddenFields.length; j++) {
                JComponent comp = (JComponent) vInput.elementAt(hiddenFields[j] + diff);
                comp.setVisible(false);
                JLabel label = (JLabel) vLabels.elementAt(hiddenFields[j] + diff);
                label.setVisible(false);    
            }
            
            for (int j = 0; j < notSearchableFields.length; j++) {
                JComponent comp = (JComponent) vInput.elementAt(notSearchableFields[j] + diff);
                comp.setVisible(false);
                JLabel label = (JLabel) vLabels.elementAt(notSearchableFields[j] + diff);
                label.setVisible(false);
            }
            diff = diff + dco.getFields().length;
        }
    }
            
    /**
     * Returns the values on which will be searched (the search condition)
     */
    public DCObject getSearchValues() {
        DCObject[] objects = new DCObject[modules.length]; 
        
        int diff = 0;
        for (int j = 0; j < modules.length; j++) {
            DCObject obj = modules[j].getDataCrowObject();
            
            final Object[] values = obj.getValues();
            int[] visibleFields = obj.getVisibleFields();
            int[] fieldTypes = obj.getFieldTypes();
    
            for (int i = 0; i < visibleFields.length; i++) {
                int fieldType = fieldTypes[visibleFields[i]];
                JComponent component = (JComponent) vInput.elementAt(visibleFields[i]  + (diff));
                
                if (fieldType == ComponentFactory._LONGTEXTFIELD) {
                    values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                    ComponentFactory._SHORTTEXTFIELD,
                                                    component);
                } else {
                    values[visibleFields[i]] = ComponentFactory.getComponentValue(
                                                    fieldType,
                                                    component);    
                }
            }
            
            for (int i = 0; i < values.length; i++) {
                if (obj.isSearchable(i)) {
                	String s = values[i] == null ? "" : values[i].toString();
                	if (s != null && !s.equals("")) {
                		obj.setValue(i, values[i]);
                	}
                }
            }
            
            objects[j] = obj;
            diff = diff + obj.getFields().length;
        }
        
        if (objects.length > 1) {
        	objects[0].addChild(objects[1]);
        }
        return objects[0];
    }
    
    /**
     * Creates the panel
     */
    private void buildPanel() {
        setLayout(Layout.getGBL());
        
        int length = 0;
        for (int i = 0; i < modules.length; i++) {
            length += modules[i].getFieldCount();
        }
        
        module = new int[length];
        
        int y = 0;
        for (int i = 0; i < modules.length; i++) {
            DCObject dco = modules[i].getDataCrowObject();
            int[] fields = modules[i].getDataCrowObject().getFieldIndices();
            
            for (int j = 0; j < fields.length; j++) {
                int field = fields[j];
                JLabel label = ComponentFactory.getLabel(dco.getLabel(field));
                
                JComponent input = null;
                if (dco.getFieldType(field) == ComponentFactory._LONGTEXTFIELD) {
                    input = ComponentFactory.getComponent(
                            ComponentFactory._SHORTTEXTFIELD, 
                            dco.getLabel(field),
                            dco.getMaxFieldLength(field)); 
                } else {
                    input = ComponentFactory.getComponent(
                            dco.getFieldType(field), 
                            dco.getLabel(field),
                            dco.getMaxFieldLength(field)); 
                }
                
                input.setPreferredSize(new Dimension(200,20));
                label.setPreferredSize(new Dimension(75,20));

                int bottom = 0;
                if (i == fields.length -1) {
                    bottom = 0;   
                }

                add( label, Layout.getGBC( 0, y, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0,  5, bottom, 5), 0, 0));
                add( input, Layout.getGBC( 1, y, 1, 1, 2.0, 2.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                            new Insets(0, 5, bottom,  5), 0, 0));

                vLabels.addElement(label);
                vInput.addElement(input);
                
                // not the real module index, but the index for the modules array
                module[y] = i;
                y++;
            }
        }
    }    
}
