/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.LinkedList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.da.ExportToPdfDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.dialogs.SaveQuestionBox;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.Requestor;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.processes.DataTask;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.ext.util.Resources;

/**
 * Base for all search panels (for objects like Software, Mp3 and Movie).
 * Has an implemenation for supporting a child panel (in case of an Object
 * with children). The functionality is not fully implemented in this class
 * as only (for now) the Audio Track and Audio CD panel have a parent-child
 * relation. 
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public abstract class TableSearchPanel extends TablePanel {

    protected SearchOptionsPanel panelSearchOpts;
    protected TableSearchConditionsPanel panelSearchConds;
    protected JPanel panelSearchActs;
    protected JScrollPane scrollSearchConds;
    
    protected TableSearchPanel childPanel;
    
    protected JButton buttonSearch;
    
    private String parentID = "";
    
    /**
     * Initializes the panel. Should be called by the child by calling
     * super(module). The popup menu and the action panel are automatically
     * created.
     * @param module the module for which this panel is created
     * @param title the title of this panel
     * @param icon the icon of this panel
     */
    public TableSearchPanel(DCModule module, String title, ImageIcon icon) {
        super(module, title, icon);

        boolean imdb = getModule().isImdbUpdateAllowed();
        boolean amazon = getModule().isAmazonUpdateAllowed();
        boolean open = this instanceof SoftwareCategoryForm.CatagorySearchPanel ?
                       false : true;

        panelSearchOpts = new SearchOptionsPanel();
        panelSearchConds = new TableSearchConditionsPanel(getModule());

        JComponent[] extraComponents = getAllExtraComponents();
        
        panelTableActions = new TableActionsPanel(false, extraComponents);
        panelTableActions.addSaveActionListener(new SaveUpdatedValuesAction());
    
        popupMenu = new PopupMenu(imdb, amazon, true, false, open, true);
        popupMenu.addAmazonActionListener(new AmazonUpdateAction());
        popupMenu.addIMDBActionListener(new ImdbUpdateAction());
        popupMenu.addSaveActionListener(new SaveSelectedValuesAction());
        popupMenu.addDeleteActionListener(new DeleteValuesAction());
        
        buildPanel();
    }
    
    /**
     * Retrieves the module specific components 
     */
    private JComponent[] getAllExtraComponents() {
        JComponent[] components = new JComponent[0];
        if (hasChild()) {
            JComponent[] childComponents = getChildModule().getSearchPanel().getExtraComponents();
            JComponent[] parentComponents = getExtraComponents();
            
            components = new JComponent[parentComponents.length + childComponents.length];
            int counter = 0; 
            for (int i = 0; i < childComponents.length; i++) {
                components[counter] = childComponents[i];
                counter++;
            }
            for (int i = 0; i < parentComponents.length; i++) {
                components[counter] = parentComponents[i];
                counter++;
            }
        }
        return components;
    }

    public abstract JComponent[] getExtraComponents();
    
    /**
     * When this search panel is acting as a child for another panel the ID
     * of the parent record for which records are shown in the child panel
     * is stored. 
     * @param sID parent ID
     */
    public void setParentID(String sID) {
        parentID = sID;
    }
    
    /**
     * Returns the parent ID for which records are shown (in case this is
     * a child panel)
     */
    public String getParentID() {
    	return parentID;
    }
    
    /**
     * Clears this table and the child table
     */
    public void clearTable() {
    	super.clearTable();
        
        if (hasChild()) {
        	childPanel.clearTable();
            childPanel.setParentID(null);
        }
    }
    
    /**
     * Fills the table with a list of data crow objects.
     * If there is child panel, the records for the selected parent
     * are retrieved as well.
     */
    public void fillTable(LinkedList data) {
        if ((data == null || data.size() == 0) && hasChild()) {
            // no data, set the parent ID for the child panel to null
            childPanel.setParentID(null);
        } else if ((data != null || data.size() > 0) && hasChild()) {
            // as the first record will be sleected by default after filling
            // the table, th child panel must be notified
        	String sID = ((DCObject) data.getFirst()).getID();
            childPanel.setParentID(sID);
        }
        
    	super.fillTable(data);
        loadChildValues();
    }
    
    /**
     * Removes the object with the given ID from the cache
     * @param sID Object ID
     */
    public void removeFromCache(String sID) {
    	table.removeFromCache(sID);
    }
    
    /**
     * Opens the recordform
     */
    public void openRecord() {
        cancelEdit();
        int row = table.getSelectedRow();
        if (row == -1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowToOpen),
                           MessageBox._WARNING);
        } else {
            DCObject dco = getObjectForRow(row);
            openRecord(dco, true);
        }
    }
    
    /**
     * Creates a PDF report. All visible fields are reported on.
     * A file browser is opened in which the user can indicate where
     * the pdf file should be stored.
     */
    public void createPdfReport() {
        DCObject[] objects = new DCObject[table.getRowCount()];
        for (int i = 0; i < objects.length; i++) {
        	objects[i] = getObjectForRow(i); 
        }
        
        ExportToPdfDialog dialog = new ExportToPdfDialog(objects);
        dialog.setVisible(true);
    }     
    
    /**
     * Permanently removes a record / records from the database.
     * The user must always confirm the delete action first before 
     * it is executed !
     */
    public void deleteValues() {
        if (!isTaskRunning()) {
            int[] rows = table.getSelectedRows();
            task = new DeletingThread(rows, getClass());
            task.start();
        } else {
            new MessageBox(Resources.getText(DCMessages.msgJobRunning),
            		       MessageBox._WARNING);    
        }
    }    

    /**
     * Saves the selected values
     */
    public void saveSelectedValues() {
        if (!isTaskRunning()) {
            int[] rows = table.getSelectedRows();
            task = new SavingThread(rows, getClass());
            task.start();
        } else {
            new MessageBox(Resources.getText(DCMessages.msgJobRunning),
                           MessageBox._WARNING);    
        }
    }
    
    /**
     * Saves all the changed values. The orginal values are stored 
     * in the 'cache' hashtable.
     * @param threaded indicates if the save should be done threaded or in the
     *                 ui thread
     */
    public void saveChangedValues(boolean threaded) {
        if (!isTaskRunning()) {
            int[] rows = table.getChangedObjectsIndices();
            task = new SavingThread(rows, getClass());
            if (threaded) {
                task.start();   
            } else {
            	task.run();
            }
        } else {
            new MessageBox(Resources.getText(DCMessages.msgJobRunning),
            		       MessageBox._WARNING);    
        }
    }
    
    /**
     * Returns a data crow object for the selected row.
     * Child objects are added to the parent data crow object. 
     * @param row the row for which a data crow object should be created
     */
    public DCObject getObjectForRow(int row) {
        String sID = table.getValueAt(row, 0, true).toString();
        DCObject dco = table.getObjectFromCache(sID);
        
        if (dco == null) {
            dco = table.getObjectForRow(row, true);
        }
        
        if (dco != null) {
            dco.setUIRowNumber(row);
            dco.setBatch(true);
            dco.setSilence(true);              
            
            Object[] oldVals = dco.getValues();
            
            DCObject dcoNew = table.getObjectForRow(row, true);
            Object[] newVals = dcoNew.getValues();
            for (int j = 0; j < newVals.length; j++) {
                if (!oldVals[j].equals(newVals[j])) {
                    dco.setValue(j, newVals[j]);
                }
            }                
            
            if (hasChild()) {
                DCObject[] children = childPanel.getObjects();
                dco.removeChildren();
                for (int j = 0; j < children.length; j++) {
                    dco.addChild(children[j]);
                }
            }                    
        }  
        return dco;
    }  
    
    /**
     * Retrieves all changed objects
     * Note: use carefully. In case of an update all this might mean that thousands 
     * of records are loaded into memory ! Right now this method is solely used for child panels
     */
    public DCObject[] getChangedObjects() {
        return table.getChangedObjects();
    }

    /**
     * Searches for the Software objects by using the conditions and options
     * as specified.
     */
    public void searchValues() {
        DCObject dco = panelSearchConds.getSearchValues();
        
        Requestor requestor = new Requestor(this.getClass(), "fillTable", null);
        dco.addRequestor(requestor);

        dco.load(panelSearchOpts.getQueryOptions());
        setStatus(Resources.getText(DCMessages.msgSearchHasBeenExecuted));
        
        if (hasChild()) {
        	childPanel.clearTable();
        }
    }
    
    /**
     * Removes the specified rows from the UI table
     */
    public void removeRows(int[] rows) {
    	super.removeRows(rows);
        if (hasChild()) {
        	childPanel.clearTable();
        }
    }
    
    /**
     * Removes hidden fields from the search table.
     */
    public void removeHiddenFields(int[] indices) {
        super.removeHiddenFields(indices);
        panelSearchConds.removeHiddenFields();
    }
    
    /**
     * Indicates whether all changes to the UI table have been saved
     * to the database
     */
    public boolean changesSaved() {
        return table.noChanges();
    } 

    /**
     * Retrieves children for the selected parent object. The children are retrieved
     * by the ID of the parent. 
     */
    public void saveChildRecords() {
        if (hasChild()) {
            int row = table.getSelectedRow();
            if (isNewlyAddedRow(row)) {
                childPanel.setParentID(null);
                childPanel.clearTable();
            } else if (!isNewlyAddedRow(row)) {
                childPanel.cancelEdit();
                if (!childPanel.changesSaved()) {
                	QuestionBox qstBox = new QuestionBox(Resources.getText(DCMessages.msgChildrenNotSaved));
                    if (QuestionBox.result.equals(Boolean.TRUE)) {
                        childPanel.saveChangedValues(false);
                    }
                }
            }
        }
    }
    
    /**
     * Tells the child panel to load children for the current row
     */
    public void loadChildValues() {
        if (hasChild()) {
            DCObject dcoChild = getChildModule().getDataCrowObject();
            int selectedRow = table.getSelectedRow();
            
            if (selectedRow > -1) {
                // Pass the ID of the parent to the child panel
                String ID = table.getValueAt(selectedRow, 0, true).toString();
                childPanel.setParentID(ID);
                
                dcoChild.setValue(dcoChild.getParentReferenceFieldIndex(), ID);
                
                // Ask the child panel to load and display the data
                childPanel.clearTable();
                dcoChild.addRequestor(new Requestor(childPanel.getClass(), "fillTable", null));
                
                QueryOptions options = new QueryOptions("2", true, true);
                dcoChild.load(options);
            }
        }
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        
        if (panelSearchOpts != null) {
            panelSearchOpts.setBackground(color);
            panelSearchOpts.checkPrecise.setBackground(color);
            panelSearchOpts.checkComplyAll.setBackground(color);
        }
        
        if (scrollSearchConds != null) {
            scrollSearchConds.setBackground(color);
            panelSearchConds.setBackground(color);
        }
    }    
    
    /**
     * Indicates whether the record is newly added or is an existing record
     */
    protected abstract boolean isNewlyAddedRow(int row);    

    /**
     * Initializes the panel
     */
    private void buildPanel() {
        JPanel panelStatus     = new JPanel(false);
        JPanel panelResult     = new JPanel(false);
        panelSearchActs = new JPanel(false);

        //**********************************************************
        //Search action panel
        //**********************************************************
        buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblSearch));
        buttonSearch.setIcon(IconLibrary._icoFetch);
        buttonSearch.setToolTipText(Resources.getText(DCTooltips.tpSearch));
        buttonSearch.addActionListener(new SearchButtonAction());
        buttonSearch.setMnemonic(KeyEvent.VK_F);
        buttonSearch.requestFocus();
 
        panelSearchActs.add( buttonSearch,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets( 0, 5, 0, 5), 0, 0));

        //**********************************************************
        //Search result panel
        //**********************************************************
        panelResult.setLayout(Layout.getGBL());

        table = ComponentFactory.getDCTable(getModule(), false);
        table.addMouseListener(tableMouseListener);

        JScrollPane scrollResults = new JScrollPane(table);
        scrollResults.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollResults.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
        
        panelResult.add(scrollResults,  Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));
        
        if (hasChild()) {
        	DCModule module  = getChildModule();
            childPanel = module.getSearchPanel();
            
            table.getSelectionModel().addListSelectionListener(new RowSelectionListener());
            
            JScrollPane scrollResults2 = new JScrollPane(childPanel.table);
            scrollResults2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            scrollResults2.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);            
            
            panelResult.add(    scrollResults2,  Layout.getGBC( 0, 1, 1, 1, 80.0, 80.0
                               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                new Insets(5, 5, 5, 5), 0, 0));
        }

        //**********************************************************
        //Status panel
        //**********************************************************
        panelStatus.setLayout(Layout.getGBL());
        labelStatus = ComponentFactory.getLabel(Resources.getText(DCLabels.lblVersion));

        panelStatus.add(labelStatus,       Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                        new Insets(0, 0, 0, 0), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());
        
        panelResult.setBorder(ComponentFactory.getTitleBorder(
                              Resources.getText(DCLabels.lblSearchResult)));
        panelStatus.setBorder(ComponentFactory.getTitleBorder(
                              Resources.getText(DCLabels.lblStatus)));
        panelSearchOpts.setBorder(ComponentFactory.getTitleBorder(
                                  Resources.getText(DCLabels.lblSearchOpts)));

        scrollSearchConds = new JScrollPane(panelSearchConds);
        scrollSearchConds.setBorder(ComponentFactory.getTitleBorder(
                         Resources.getText(DCLabels.lblSearchCond)));

        scrollSearchConds.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollSearchConds.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        
        add(    scrollSearchConds,Layout.getGBC( 0, 0, 1, 2, 20.0, 20.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchOpts,  Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelSearchActs,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelResult,      Layout.getGBC( 0, 3, 3, 1, 100.0, 100.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelTableActions,Layout.getGBC( 0, 4, 3, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 5, 3, 1, 1.0, 1.0
               ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(table);
    }

    /**
     * Search Option panel. Here options, such as 'comply to all conditions', 
     * are maintained
     */
    public class SearchOptionsPanel extends JPanel {
        private JCheckBox checkComplyAll = 
            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAndOrSelect));
        private JCheckBox checkPrecise =
            ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblPreciseSelect));
        private JLabel labelOrderBy = ComponentFactory.getLabel(
                                      Resources.getText(DCLabels.lblOrderBy));
        private JComboBox comboOrderBy = ComponentFactory.getComboBox();

        public SearchOptionsPanel() {
            DCObject o = getModule().getDataCrowObject();
            setLayout(Layout.getGBL());

            String[] colNames    = o.getLabels();
            comboOrderBy.addItem("");
            for (int i = 0; i < colNames.length; i++) {
                comboOrderBy.addItem(colNames[i]);
            }
            
            comboOrderBy.setSelectedItem(colNames[1]);
            checkComplyAll.setSelected(true);
            
            add( checkComplyAll, Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( checkPrecise,   Layout.getGBC( 0, 1, 2, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 3, 5), 0, 0));
            add( labelOrderBy,   Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 0, 5), 0, 0));
            add( comboOrderBy,   Layout.getGBC( 1, 2, 1, 1, 5.0, 5.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets( 0, 5, 5, 5), 0, 0));
        }

        /**
         * Returns the query options used for searching values
         */
        public QueryOptions getQueryOptions() {
            boolean bPreciseSelect = checkPrecise.isSelected();
            boolean bComplyToAllCondition = checkComplyAll.isSelected();
            String sOrderBy = "";
            if (comboOrderBy.getSelectedIndex() > 0) {
                sOrderBy = getModule().getFieldDefinition(comboOrderBy.getSelectedIndex() - 1).getDatabaseFieldName();
            }
            return new QueryOptions(sOrderBy, bComplyToAllCondition, bPreciseSelect);
        }
    }    
    
    // ****************************************************************************
    // ACTIONS
    // ****************************************************************************
    private class DeletingThread extends DataTask {

        public DeletingThread(int[] rows, Class clazz) {
        	super(rows, clazz);
        }
        
        public void run() {
            try {
                startRunning();
                cancelEdit();
                denyActions();
                
                int[] rows = table.getSelectedRows();
    
                if (rows == null || rows.length <= 0) {
                    new MessageBox(Resources.getText(DCMessages.msgNoRowsSelected),
                                   MessageBox._WARNING);
                    stopRunning();
                } else {
                    new QuestionBox(Resources.getText(DCMessages.msgDeleteQuestion));
                    if (!QuestionBox.result.booleanValue()) {
                        stopRunning();
                    }
                }
    
                for (int i = rows.length -1; i > -1 && keepOnRunning(); i--) {
                    DCObject dco = getObjectForRow(rows[i]);
                    dco.addRequestor(new Requestor(clazz, "removeRows",
                                                   new Object[] {new int[] {rows[i]}}));
                    dco.clearValues();
                    dco.undoChanges();
                    dco.setValue(DCObject._ID, table.getValueAt(rows[i], 0, true));
                    
                    dco.setEndOfBatch(i == 0);
                    dco.delete(); 
                    
                    try {
                    	sleep(300);
                    } catch (Exception ignore) {} // doesn't want to sleep..
                }
                
                setStatus(Resources.getText(DCMessages.msgDeleteHasBeenExecuted));
            } finally {
            	stopRunning();
            	allowActions();
            }
        }
    }    
    
    private class SavingThread extends DataTask {
        public SavingThread(int[] rows, Class clazz) {
            super(rows, clazz);
        }
        
        public void run() {
            try {
                startRunning();
                cancelEdit();
                denyActions();
                
                if (hasChild() && !childPanel.changesSaved() && ((rows == null || rows.length == 0))) {
                	rows = table.getSelectedRows();
                } 
                
                if (rows == null || rows.length == 0) {
                    stopRunning();
                }
                
                setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
    
                boolean ignoreErrors = false;
                for (int i = rows.length - 1; i > -1 && keepOnRunning(); i--) {
                    DCObject dco = table.getChangedObject(rows[i]);

                    if (dco == null) {
                        dco = table.getObjectForRow(rows[i], true);
                    }
                    
                    dco.removeChildren();
                    if (hasChild()) {
                        DCObject[] children = childPanel.getChangedObjects();
                        for (int j = 0; j < children.length; j++) {
                            DCObject child = children[j];
                            child.addRequestor(new Requestor(childPanel.getClass(), "removeFromCache", 
                                                             new Object[] {child.getID()}));
                            dco.addChild(child);
                        }
                    }
                    
                    dco.setSilence(true);
                    dco.setBatch(true);
                    dco.setEndOfBatch(i == 0);
                    dco.addRequestor(new Requestor(clazz, "removeFromCache", 
                    		                       new Object[] {dco.getID()}));
                    try {
                        dco.saveUpdate();
                    } catch (ValidationException vExp) {
                        if (!ignoreErrors) {
                            SaveQuestionBox question = new SaveQuestionBox(vExp);
                            if (question.getResult() == SaveQuestionBox._CANCEL) {
                                stopRunning();
                            } else if (question.getResult() == SaveQuestionBox._IGNORE) {
                                ignoreErrors = true;
                            }
                        }
                    }
                    
                    try {
                        sleep(300);
                    } catch (Exception ignore) {} // doesn't want to sleep..
                }
                setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
            } finally {
                allowActions();
            	stopRunning();
            }
            
            if (hasChild()) {
            	childPanel.saveChangedValues(false);
            }
        }
    }    
    
    private class RowSelectionListener implements ListSelectionListener {
    	public void valueChanged(ListSelectionEvent e){
            if (e.getValueIsAdjusting()) {
                saveChildRecords();
            	loadChildValues();
            }
    	}
    }
    
    public class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            searchValues();
        }
    }

    private class SaveSelectedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveSelectedValues();
        }
    }

    private class AmazonUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    private class SaveUpdatedValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveChangedValues(true);
        }
    }

    private class DeleteValuesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteValues();
        }
    }

    private class ImdbUpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancelEdit();
            startIMDBUpdate();
        }
    }
}
