/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.renderers.ComboBoxTableCellRenderer;
import net.sf.dc.console.components.renderers.DCTableCellRenderer;
import net.sf.dc.console.components.renderers.DCTableHeaderRenderer;
import net.sf.dc.console.components.renderers.DCTableHeaderRendererRequired;
import net.sf.dc.console.components.renderers.DCTableRowIndexCellRenderer;
import net.sf.dc.console.components.renderers.PictureTableCellRenderer;
import net.sf.dc.console.components.renderers.TimeFieldTableCellRenderer;
import net.sf.dc.console.dialogs.FieldSelectorDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.UpdateAllDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.StringUtils;

/**
 * Table implementation with caching of changed values.
 * Can either be initialized as a simple table or a table for module (with
 * all the correct components and headers)
 *
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class DCTable extends JTable {
    
    private final DCModule module;
    private final DCTableModel tableModel;
    private final Hashtable cache = new Hashtable();

    private final TableValueChangedAction tableChangeListener = 
        new TableValueChangedAction();    
    
    private final boolean caching;
    private final boolean readonly;
    
    private Vector vHiddenColumns = new Vector();
    private Vector vColumns = new Vector();
    
    private TableColumn rowIndexColumn;

    /**
     * Initializes a very simple table
     */
    public DCTable() {
        super(new DCTableModel());
        createRowIndexColumn();
        addRowIndexColumn();
        setProperties();
        module = null;
        tableModel = (DCTableModel) getModel();
        caching = false;
        readonly = false;
    }

    /**
     * Initializes a table for a specific module
     * @param module the mocule for which this table is created
     * @param readonly allow / block changes 
     */
    public DCTable(DCModule module, boolean readonly) {
        super(new DCTableModel(module));
        tableModel = (DCTableModel) getModel();
        caching = module != null;
        this.module = module;
        this.readonly = readonly;
        
        setListeningForChanges(true);
        
        DCObject dco = module.getDataCrowObject();
        
        String[] headers = dco.getLabels();
        int[] maxLength = dco.getMaxFieldLengths();
        int[] fieldTypes = dco.getFieldTypes();

        tableModel.setColumnCount(fieldTypes.length);

        JTextField text = ComponentFactory.getTextField();
        for (int i = 0; i < fieldTypes.length; i++) {
            TableColumn columnNew = getColumnModel().getColumn(i);
            columnNew.setHeaderValue(headers[i]);

            JComponent comp = ComponentFactory.getComponent(fieldTypes[i],
                                           headers[i],
                                           maxLength[i]);
            comp.setAutoscrolls(false);
            comp.setBorder(null);
            comp.setDoubleBuffered(true);
            comp.setIgnoreRepaint(true);
            comp.setVerifyInputWhenFocusTarget(false);
            
            if (fieldTypes[i] == ComponentFactory._LONGTEXTFIELD) {
                comp = ComponentFactory.getComponent(
                                ComponentFactory._SHORTTEXTFIELD,
                                headers[i],
                                maxLength[i]);
            }
            
            if (i == 0) {
                text.setEditable(false);
                text.setEnabled(false);
                text.setFont(ComponentFactory.getBoldFont());
                text.setForeground(ComponentFactory.getDisabledColor());
                columnNew.setCellEditor(new DefaultCellEditor(text));
                DCTableCellRenderer renderer = new DCTableCellRenderer();
                renderer.setFont(ComponentFactory.getBoldFont());
                columnNew.setCellRenderer(renderer);
            } else {
                switch (fieldTypes[i]) {
                    case ComponentFactory._NUMBERFIELD:
                    case ComponentFactory._LONGTEXTFIELD :
                    case ComponentFactory._SHORTTEXTFIELD :
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));
                        break;
                    case ComponentFactory._TIMEFIELD :
                        TimeFieldTableCellRenderer timeRenderer = new TimeFieldTableCellRenderer();
                        NumberField numberField = ComponentFactory.getNumberField();
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) numberField));
                        columnNew.setCellRenderer(timeRenderer);
                        break;
                    case ComponentFactory._URLFIELD :
                        columnNew.setCellEditor(new DefaultCellEditor((JTextField) comp));
                        DCTableCellRenderer renderer = new DCTableCellRenderer();
                        renderer.setForeground(new Color(0, 0, 255));
                        columnNew.setCellRenderer(renderer);
                        break;
                    case ComponentFactory._PICTUREFIELD :
                        text = ComponentFactory.getTextField();
                        text.setEditable(false);
                        text.setFont(ComponentFactory.getUnreadableFont());
                        columnNew.setCellEditor(new DefaultCellEditor(text));
                        columnNew.setCellRenderer(
                                new PictureTableCellRenderer(module.getIndex()));
                        break;                        
                    case ComponentFactory._SOFTWARECATEGORYCOMBO:
                        columnNew.setCellRenderer(new ComboBoxTableCellRenderer());
                        columnNew.setCellEditor(new DefaultCellEditor((JComboBox) comp));
                        break;
                    case ComponentFactory._MP3GENRECOMBO:
                    case ComponentFactory._YEARCOMBO:
                    case ComponentFactory._YESNOCOMBO:
                        columnNew.setCellEditor(new DefaultCellEditor((JComboBox) comp));
                        break;
                }    
            	comp.setEnabled(!readonly);
            }
        }

        createRowIndexColumn();
        addRowIndexColumn();
        setProperties();
    }
    
    /**
     * Indicates if the table data can be edited
     */
    public boolean isReadOnly() {
    	return readonly;
    }
    
    /** 
     * Indicates if changes have been made to the data of the table
     */
    public boolean noChanges() {
        return cache.size() == 0;
    }    
    
    /**
     * (Re-) applies all headers
     */
    public void applyHeaders() {
    	removeRowIndexColumn();
        for (Enumeration enum = getColumnModel().getColumns(); enum.hasMoreElements(); ) {
            TableColumn column = (TableColumn) enum.nextElement();
            column.setHeaderRenderer(getTableHeaderRenderer());
            vColumns.addElement(column);
        }
        addRowIndexColumn();
    }
    
    /**
     * All columns to be removed are still visible in the table. The columns 
     * are retrieved, stored in a Vector and then removed from the view
     * @param hiddenColumns The model indices of the columns
     */
    public void removeHiddenFields(int[] hiddenColumns) {
        cancelEdit();
        resetTable();
        
        vHiddenColumns.clear();
        removeRowIndexColumn();
        
        for (int i = 0; i < hiddenColumns.length; i++) {
            int tableIndex = convertColumnIndexToView(hiddenColumns[i]);
            TableColumn column = getColumnModel().getColumn(tableIndex);
            vHiddenColumns.addElement(column); 
            removeColumn(column);
        }
        addRowIndexColumn();
        setRequiredFields();
    }
    
    /**
     * Clears the cached objects
     */
    public void clearCache() {
    	cache.clear();
    }
    
    /**
     * Removes the object with the given ID from the cache (unmark as changed)
     * @param sID object ID
     */
    public void removeFromCache(String sID) {
    	cache.remove(sID);
    }
    
    /**
     * Returns all Data Crow Objects currently persisting in the (UI) table
     */
    public DCObject[] getObjects() {
        DCObject[] objects = new DCObject[tableModel.getRowCount()];
        for (int i = 0; i < objects.length; i++) {
            objects[i] = getObjectForRow(i, true);
        }
        return objects;
    }   
    
    /**
     * Returns the object with the given ID from the cache
     * @param sID Unique object ID
     */
    public DCObject getObjectFromCache(String sID) {
        return (DCObject) cache.get(sID);
    }
    
    /**
     * Returns all the values at the give row
     * @param row the UI row number
     * @param hidden indicates if values of hidden fields should also be returned 
     */
    public DCObject getObjectForRow(int row, boolean hidden) {
        cancelEdit();
        
        if (row == -1) {
        	new MessageBox(Resources.getText(DCMessages.msgNoRowsSelected), MessageBox._WARNING);
            return null;
        }
        Object[] values = hidden ? new Object[tableModel.getColumnCount()] :
                                   new Object[getColumnCount()];
        for (int j = 0; j < values.length; j++) {
            Object o = hidden ? tableModel.getValueAt(row, j) :
                                getValueAt(row, j);
            values[j] = o == null ? "" : o;
        }
        
        DCObject dco = module.getDataCrowObject();
        dco.setValues(values);
        return dco;
    }
    
    /**
     * Retrieve all changed objects. Use with care, this could be heavy on the
     * system when many rows have been changed. Consider using the method
     * getChangedObjectsIndices
     */
    public DCObject[] getChangedObjects() {
        DCObject[] objects = new DCObject[cache.size()];
        int counter = 0;
        for (Enumeration enum = cache.keys(); enum.hasMoreElements(); counter++) {   
            String sID = enum.nextElement().toString();
            DCObject obj = (DCObject) cache.get(sID);
            objects[counter] = obj;
        }
        return objects;
    }    
    
    /**
     * Retrieves the rownumbers of all changed objects
     */
    public int[] getChangedObjectsIndices() {
        int[] rows = new int[cache.size()];
        int counter = 0;
    	for (Enumeration enum = cache.keys(); enum.hasMoreElements(); counter++) {   
            String sID = enum.nextElement().toString();
            int row = getRowNumberWithID(sID);
            rows[counter] = row;
        }
        return rows;
    }

    /**
     * Retrieve a changed object (to get the row indices, use getChangedObjectIndices)
     * @param index row index
     */
    public DCObject getChangedObject(int index) {
        String sID = getValueAt(index, 0, true).toString();
        DCObject dco = (DCObject) cache.get(sID);
        if (dco != null) {
        	dco.setUIRowNumber(index);
        }
        return dco;
    }
    
    /**
     * Gets a value from the model (hidden or visible) or the table 
     * (always visible)
     * @param row row index
     * @param col column index
     * @param hidden get the value from the model or table
     */
    public Object getValueAt(int row, int col, boolean hidden) {
        if (hidden) {
        	return tableModel.getValueAt(row, col);
        } else {
            return getValueAt(row, col);
        }
    }
    
    /**
     * Clear the table of all values. Clear the cache as well.
     */
    public void clearTable() {
        cache.clear();
        cancelEdit();

        setListeningForChanges(false);
        tableModel.setRowCount(0);
        setListeningForChanges(true);
    }
    
    /**
     * Returns the row number of the record with the given ID
     * (product object ID; ie. the value of Software._ID)
     * @param ID
     */
    public int getRowNumberWithID(String ID) {
        cancelEdit();
        String sID = "";
        for (int i = 0; i < tableModel.getRowCount(); i++) {
            sID = tableModel.getValueAt(i, 0).toString();
            if (sID.equals(ID)) {
                return i;
            }
            sID = null;
        }
        return -1;
    }  
    
    /**
     * Sets the amount of columns to be displayed
     * @param count columns amount
     */
    public void setColumnCount(int count) {
    	tableModel.setColumnCount(count);
    }
    
    /**
     * Remove the rows from the table with the given indices
     */
    public void removeRows(int[] rows) {
        cancelEdit();
        for (int i = rows.length -1; i > -1; i--) {
            tableModel.removeRow(rows[i]);
        }
    }   
    
    /**
     * Removes the selected or the last row from the UI table
     */
    public void removeRow() {
        int[] rows = getSelectedRows();
        if (rows.length == 0) {
            tableModel.removeRow(tableModel.getRowCount() -1);
        } else {
            for (int i = rows.length; i > 0; i--) {
                tableModel.removeRow(rows[i-1]);
            }
        }     
    }    
    
    /**
     * Removes a row from the UI table
     * @param row row index
     */
    public void removeRow(int row) {
        tableModel.removeRow(row);
    }    
    
    /**
     * Sets the amount of rows to be displayed
     * @param count rows amount
     */
    public void setRowCount(int count) {
        tableModel.setRowCount(count);
    }    
    
    /**
     * Adds a row to the table
     * @param data values to be added
     */
    public void addRow(Object[] data) {
        tableModel.addRow(data);
        removeColumnSelectionInterval(0,0);
        setSelectedRow(getRowCount() -1);
    }  
    
    public void setSelectedRow(int row) {
        if (getSelectedRow() > -1) {
            removeColumnSelectionInterval(0, getColumnCount() -1);
            removeRowSelectionInterval(getSelectedRow(), getSelectedRow());
        }
            
        addRowSelectionInterval(row, row);
        addColumnSelectionInterval(0, getColumnCount() -1);
    }
    
    /**
     * Adds the datacrow objects to the table
     * @param data  datacrow objects
     * @param addUIRowID indicates whether a row ID should be displayed
     */
    public void addRow(LinkedList data, boolean addUIRowID) {
        while (data.size() > 0) {
            DCObject lcObj = (DCObject) data.removeFirst();
            Object[] row = lcObj.getValues();
            
            if (addUIRowID && (row[0] == null || row[0].equals(""))) {
                row[0] = "" + (tableModel.getRowCount() + 1);
            }
            addRow(row);
        }
        data.clear();
    }    
    
    /**
     * Updates the row with the given ID with the data (values)
     * @param ID the ID (Product Object ID)
     * @param values the new values
     */
    public void updateRow(String ID, Object[] values) {
        cancelEdit();

        int row = getRowNumberWithID(ID);
        if (row != -1) {
            for (int j = 0; j < values.length; j++) {
                tableModel.setValueAt(values[j], row, j);
                addRowToCache(row, j);
            }
        }
    }    
    
    /**
     * Updates a row at the given rownumber. If indicated this will
     * update existing data.
     * @param rowNumber UI row to be updated
     * @param rowData the data to be written to the row
     * @param overWrite indicates of existing data should be overwritten
     */
    public void updateRowAt(int rowNumber,
                            Object[] rowData,
                            boolean overWrite) {
        cancelEdit();

        int[] fieldIndices = module.getDataCrowObject().getFieldIndices();
        for (int i = 0; i < rowData.length; i++) {
            Object oNew = rowData[i];
            Object oExist = tableModel.getValueAt(rowNumber, fieldIndices[i]);

            if (!overWrite && (oExist != null &&
                               !oExist.toString().trim().equals(""))) {
                oNew = oExist;
            }

            if (oNew == null || oNew.toString().trim().equals("")) {
                oNew = oExist;
            }
            tableModel.setValueAt(oNew, rowNumber, fieldIndices[i]);
            addRowToCache(rowNumber, fieldIndices[i]);
        }
    } 
    
    /**
     * Add a changed row to the cache. Can also be used to force save records
     * (add a unchanged row to the cache) 
     * @param row changed row number
     * @param column changed column number
     */
    public void addRowToCache(int row, int column) {
        setListeningForChanges(false);
        
        if (row != -1 && column != -1) {
            String sID = tableModel.getValueAt(row, 0).toString();
            
            if (sID != null && !sID.equals("")) {
                DCObject dco;
                if (!cache.containsKey(sID)) {
                    dco = getObjectForRow(row, true);
                    dco.undoChanges();
                } else {
                    dco = (DCObject) cache.get(sID);
                }
                dco.setValue(column, tableModel.getValueAt(row, column));
                cache.put(sID, dco);
            }
        }        
        setListeningForChanges(true);
    }      
    
    /**
     * Fills the table with the specified values
     * @param data list containing the rows of data to be displayed
     */
    public void fillTable(LinkedList data) {
        int selectedRow = getSelectedRow();
        int selectedCol = getSelectedColumn();
        
        clearTable();

        tableModel.getDataVector().clear();
        clearCache();
        
        setListeningForChanges(false);
        
        while (data.size() > 0) {
            DCObject obj = (DCObject) data.removeFirst();
            Object[] values = obj.getValues();
            addRow(values);
            values = null;
            obj = null;
        }
        
        try {
            setRowSelectionInterval(selectedRow, selectedRow);
            setColumnSelectionInterval(selectedCol, selectedCol);            
        } catch (Exception ignore) {}
        
        if (getSelectedRow() < 0) {
            try {
                setRowSelectionInterval(0, 0);
                setColumnSelectionInterval(0, 0);
            } catch (Exception exp) {}
        }
        
        setListeningForChanges(true);
        
        data.clear();
        data = null;
    }    
    
    /**
     * Creates a visual representation (different header) 
     * of the required fields
     * @param o data crow object (used as a reference)
     */
    public void setRequiredFields() {
        DCObject dco = module.getDataCrowObject();
        int counter = 0;
        removeRowIndexColumn();
        for (Enumeration enum = vColumns.elements(); enum.hasMoreElements(); ) {// && counter < vColumns.size() - 1; ) {
            TableColumn column = (TableColumn) enum.nextElement();
//            if (!((String) column.getHeaderValue()).equals("Row")) {
	            if (dco.isRequired(counter)) {
	                column.setHeaderRenderer(getRequiredTableHeaderRenderer());
	            } else {
	                column.setHeaderRenderer(getTableHeaderRenderer());
	            }
//	            System.out.println(counter);
	            counter++;
//            } else {
//            	System.out.println("else");
//            }
        }
        addRowIndexColumn();
    }    
    
    /**
     * Tries to apply the current editing of the table cell.
     * If the current value cannot be applied, the editing is simply
     * cancelled (changes are discarded) 
     */
    public void cancelEdit() {
        int selectedRow = getSelectedRow();
        int selectedColumn = getSelectedColumn();
        
        if (selectedRow != -1) {
            for (int i = 0; i < getColumnCount(); i++) {
                boolean success = false;
                try {
                    getCellEditor(selectedRow, i).stopCellEditing();
                    success = true;
                } catch (Exception ignore) {}
                
                if (!success) {
                    getCellEditor(selectedRow, i).cancelCellEditing();
                }
            }
        }
    }   

    //*************************************************************************
    //Dialogs
    //*************************************************************************
    
    /**
     * Opens the fields selector dialog and applies the hidden fields on the
     * table. Also the hidden fields setting is set for the current module.
     */
    public void openFieldSelector() {
        Vector vVisibleColumns = new Vector();
        removeRowIndexColumn();
        for (Enumeration enum = getColumnModel().getColumns(); enum.hasMoreElements(); ) {
            vVisibleColumns.addElement(enum.nextElement());
        }
        FieldSelectorDialog dialog = new FieldSelectorDialog(vVisibleColumns, 
                                                             vHiddenColumns);
        addRowIndexColumn();
        dialog.setVisible(true);
        if (FieldSelectorDialog.isSuccessful()) {
            vVisibleColumns = dialog.getVisibleFields(); 
            vHiddenColumns = dialog.getHiddenFields();

            // get the hidden columns table model indices by comparing the
            // columns of the table model with the table              
            int[] modelIndices = new int[vHiddenColumns.size()];
            int counter = 0;
            for (Enumeration enum = vHiddenColumns.elements(); enum.hasMoreElements(); ) {
                TableColumn column = (TableColumn) enum.nextElement();
                modelIndices[counter++] = column.getModelIndex();
            }            
            Settings.setValue(module.getHiddenFieldsSettingsKey(), modelIndices);
            MainFrame.setHiddenFields();
        }
    }    
    
    /**
     * Opens the Update All dialog
     */
    public void openUpdateAllDialog() {
        UpdateAllDialog dialog = new UpdateAllDialog(
                                        module.getDataCrowObject(), 
                                        DataCrow.mainFrame);
        // if the update was a success
        if (dialog.isSuccess()) {
            dialog.setModal(false);
            // retrieve the data which will be applied on the table
            Object[] values = dialog.getData();
            int rows = tableModel.getRowCount();
            setListeningForChanges(false);
            for (int i = 0; i < rows; i++) {
                for (int j = 0; j < values.length; j++) {
                    // only update a field with a 'real' / not empty value
                    if (values[j] != null
                            && !StringUtils.isEmpty(values[j].toString())
                            && !values[j].toString().equals("-1")) {
                        tableModel.setValueAt(values[j], i, j);
                        addRowToCache(i, j);
                    }
                }
            }
            setListeningForChanges(true);
        }
    }     

    //*************************************************************************
    //Private methods and classes
    //*************************************************************************
    
    /**
     * Adds all the columns to the table (hidden and visible fields)
     */
    private void resetTable() {
        for (Enumeration enum = vColumns.elements(); enum.hasMoreElements(); ) {
            TableColumn column = (TableColumn) enum.nextElement();
            try {
                getColumn(column.getIdentifier());
            } catch (Exception exp) {
                addColumn(column);
            }
        }
    }      
    
    /**
     * Applies all default properties on the table
     */
    private void setProperties() {
        setBackground(ComponentFactory.getTableColor());
        setAutoscrolls(true);

        DCObject dco = module == null ? null : module.getDataCrowObject();
        
        setDefaultRenderer(Object.class, new DCTableCellRenderer());
        setFont(ComponentFactory.getStandardFont());

        setRowSelectionAllowed(true);
        setColumnSelectionAllowed(false);
        setRequestFocusEnabled(true);
        setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        setAlignmentY(JTable.TOP_ALIGNMENT);
        getTableHeader().setReorderingAllowed(true);
        setShowHorizontalLines(true);
        
        setBackground(new Color(255, 255, 255));
        setGridColor(new Color(220,220,200));
        
        applyHeaders();

        setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    }    
    
    /**
     * Enables / disables caching of changed values
     * @param b
     */
    private void setListeningForChanges(boolean b) {
        boolean enable = b && caching;
        if (enable && getListeners(tableChangeListener.getClass()).length == 0) {
            tableModel.addTableModelListener(tableChangeListener);
        } else if (!enable) {
            tableModel.removeTableModelListener(tableChangeListener);
        }
    }
    
    private DefaultTableCellRenderer getRequiredTableHeaderRenderer() {
        DCTableHeaderRendererRequired hdrRenderer = new DCTableHeaderRendererRequired();
        return hdrRenderer;
    }

    private DefaultTableCellRenderer getTableHeaderRenderer() {
        DCTableHeaderRenderer hdrRenderer = new DCTableHeaderRenderer();
        return hdrRenderer;
    }
    
    private void createRowIndexColumn() {
        rowIndexColumn = new TableColumn();
        rowIndexColumn.setHeaderValue("Row");
        rowIndexColumn.setMaxWidth(40);
        rowIndexColumn.setPreferredWidth(40);
        rowIndexColumn.setWidth(40);
        JTextField text = ComponentFactory.getTextField();
        text.setEditable(false);
        text.setEnabled(false);
        text.setBackground(Settings.getValueAsColor(Repository.settings.stColor));
        rowIndexColumn.setCellEditor(new DefaultCellEditor(text));
        DCTableRowIndexCellRenderer renderer = new DCTableRowIndexCellRenderer();
        renderer.setFont(ComponentFactory.getBoldFont());
        rowIndexColumn.setCellRenderer(renderer);
        rowIndexColumn.setHeaderRenderer(getTableHeaderRenderer());
    }
    
    private void addRowIndexColumn() {
        getColumnModel().addColumn(rowIndexColumn);
        moveColumn(getColumnCount() - 1, 0);
    }
    
    private void removeRowIndexColumn() {
    	getColumnModel().removeColumn(rowIndexColumn);
    }
    
    /**
     * Handles changes made to the table (on screen)
     */
    private class TableValueChangedAction implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            if (    e.getType() == TableModelEvent.INSERT || 
                    e.getType() == TableModelEvent.UPDATE) {
                int row = getSelectedRow();
                addRowToCache(row, e.getColumn());
            }
        }
    }    
    
    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            addColumnSelectionInterval(0, getColumnCount() -1);
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
