/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.cache;

import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JComboBox;

import net.sf.dc.core.objects.DCObject;
import net.sf.dc.db.Query;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.db.QueryWhat;
import net.sf.dc.messages.Messages;
import net.sf.dc.wf.WorkFlow;

/**
 * Caches data crow objects in memory. This prevents the application
 * from having to go to the database for regurarly used values.
 * Commonly cached objects are small and are mostly diplayed in
 * combo boxes (such as the Software Category)
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class Cache {

    private final Vector vRegUiComponents = new Vector();
    private final WorkFlow wf = new WorkFlow();
    private DCObject[] cachedObjects;
    private final DCObject o;    
    
    /**
     * Initializes the cache for a specific data crow object
     * @param o data crow object used as a reference
     */
    public Cache(DCObject o) {
        this.o = o;
        load();
    }
    
    /**
     * Updates the UI component which are registered on cache refreshes
     */
    private void updateUiComponent() {
        for (Enumeration enum = vRegUiComponents.elements(); enum.hasMoreElements(); ) {
            JComboBox combo = (JComboBox) enum.nextElement();
            try {            
                Object o = combo.getSelectedItem();
                combo.removeAllItems();
                combo.addItem("");
                
                for (int i = 0; i < cachedObjects.length; i++) {
                    combo.addItem(cachedObjects[i]);
                }
                if (o != null) {
                	combo.setSelectedItem(o);
                } else {
                	combo.setSelectedIndex(0);
                }
            } catch (Exception exp) {
                Messages.setMessage(exp);
                combo.setSelectedIndex(0);
            }
        }
    }
    
    /**
     * Register a combo box. This combo box will be updated with
     * the currently cached values. Everytime the cache is refreshed, 
     * this combo box will be refreshed as well.
     * @param combo combo box to be registered
     */
    public void registerUIComponent(JComboBox combo) {
        vRegUiComponents.addElement(combo);
        updateUiComponent();
    }
    
    /**
     * Retrieves all the cached objects
     */
    public DCObject[] getCachedObjects() {
        return cachedObjects;
    }

    /**
     * Clears the cache
     */
    public void clear() {
        cachedObjects = null;
    }

    /**
     * Retrieves the cached object with the given display value
     * (toString() value)
     * @param s display value to search on
     */
    public DCObject getObjectForDisplayValue(String s) {
        for (int i = 0; i < cachedObjects.length; i++) {
            if (cachedObjects[i].toString().equals(s)) {
                return cachedObjects[i];
            }
        }
        return null;
    }
    
    /**
     * Retrieves the cached object with the given ID
     * @param id ID to search on
     */
    public DCObject getObjectForID(String id) {
        for (int i = 0; i < cachedObjects.length; i++) {
            if (cachedObjects[i].getID().equals(id)) {
                return cachedObjects[i];
            }
        }
        return null;
    }
    
    /**
     * Retrieves the objects to be cached. All items of the 
     * data crow object type will be retrieved and added to the
     * cache.
     */
    public final void load() {
        cachedObjects = new DCObject[0];
        QueryWhat what = new QueryWhat(o.getDatabaseFieldNames(), 
                                       o.getTableName(), 
                                       o.getSelectTableDef());
        
        QueryOptions options = new QueryOptions("2", false, false);
        Query qry = new Query(what, null, options, Query._SELECT, null, null);
        LinkedList data = wf.runQueryDirect(qry);
        cachedObjects = new DCObject[data.size()];
        cachedObjects = (DCObject[]) data.toArray(cachedObjects);
        data.clear();
        data = null;
        updateUiComponent();        
    }    
}
