/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 Robert-Jan van der Waals                    *
 *                             rwaals@anytimenow.com                          *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.core.objects.RequestorCollection;
import net.sf.ext.util.StringUtils;

/**
 * Creates a query String by using the QueryOptions, the QueryWhat, the 
 * QueryConditions and the query type
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class Query {

    /** Undifined query */
    public static final int _UNDEFINED = 0;
    /** Update query */
    public static final int _UPDATE = 1;
    /** Create statement */
    public static final int _CREATE = 2;
    /** Insert query */
    public static final int _INSERT = 3;
    /** Select query */
    public static final int _SELECT = 4;
    /** Delete query */
    public static final int _DELETE = 5;
    

    private static final int _SELECTPRECISE = 6;
    private static final int _SELECTOR = 7;
    private static final int _SELECTPRECISEOR = 8;

    private static final String sDelete = "DELETE FROM ";
    private static final String sSelect = "SELECT DISTINCT ";
    private static final String sUpdate = "UPDATE ";
    private static final String sCreate = "CREATE TABLE ";
    private static final String sInsert = "INSERT INTO ";
    private static final String sFrom = "FROM ";
    private static final String sWhere = "WHERE ";
    private static final String sAnd = " AND ";
    private static final String sOr = " OR ";
    private static final String sValues = " VALUES ";
    private static final String sSet = " SET ";
    private static final String sLikeBegin = " LIKE '%";
    private static final String sLikeEnd = "%'";
    private static final String sClose = " ;";
    private static final String sOrderBy = " ORDER BY ";
    private static final String sUpper = "upper";

    private RequestorCollection requestors;
    private int iQueryType;

    private Vector vColumns;
    private String[][] conditionColumns;
    private Vector vConditionValues;

    private boolean bPreciseSelect = false;
    private boolean bComplyToAllConditions = true;

    private String sOrderByField;
    private String sTableName;
    private String sSelectTableDef;
    private String sID;

    private String sQuery = "";

    private boolean bSilence = false;

    private final StringBuffer sbCols = new StringBuffer(" ");
    private final StringBuffer sbValsDelete = new StringBuffer(" ");
    private final StringBuffer sbVals = new StringBuffer(" ");
    private final StringBuffer sbColIsVal = new StringBuffer(" ");
    private final StringBuffer sbColIsValUpd = new StringBuffer(" ");
    private final StringBuffer sbColLikeVal = new StringBuffer(" ");
    private final StringBuffer sbColLikeValOR = new StringBuffer(" ");
    private final StringBuffer sbColIsValOR = new StringBuffer(" ");
    private final StringBuffer sbOrderBy = new StringBuffer(" ");

    private boolean isBatch = false; 
    private boolean endOfBatch = false;
    
    /**
     * Creates a new Query object. In this object the condition values and
     * their fields are stored, the columns of the object, the options, the
     * query type and the requestors of the query (which might want the query
     * to result in an action)
     * The public query types are not all there is to it, depending on the
     * query options, a subtype is set (only in case of select queries)
     * From these values a query string is created
     * @param what specifies the columns and tablename
     * @param conditions the fields with their values which form the condition
     *        (can be null)
     * @param options options for the query (can be null)
     * @param iQueryType the query type (insert/update/..)
     * @param requestors actions which are past on to the requestors 
     *        (can be null)
     * @param sID  unique ID of the record
     */
    public Query(   QueryWhat what,
                    QueryCondition conditions,
                    QueryOptions options,
                    int iQueryType,
                    RequestorCollection requestors,
                    String sID) {
        
        if (conditions != null) {
            conditionColumns = conditions.getColumns();
            vConditionValues = conditions.getValues();
        }

        if (options != null) {
            bPreciseSelect = options.getPreciseSelect();
            bComplyToAllConditions = options.getComplyAllConditions();
            sOrderByField = options.getOrderByField();
        }

        if (what != null) {
            sTableName = what.getTableName();
            sSelectTableDef = what.getSelectTableDef();
            vColumns = what.getColumns();
        }

        this.sID = sID;
        this.requestors = requestors;
        this.iQueryType = iQueryType;

        if (this.iQueryType == _SELECT) {
            if (bPreciseSelect && !bComplyToAllConditions) {
                this.iQueryType = _SELECTPRECISEOR;
            } else if (bPreciseSelect) {
                this.iQueryType = _SELECTPRECISE;
            } else if (!bComplyToAllConditions) {
                this.iQueryType = _SELECTOR;
            }
        }
    }
    
    /**
     * Sets this query as being part of a batch
     * @param endOfBatch last of the batch y/n
     */
    public void setBatch(boolean endOfBatch) { 
        this.isBatch = true; 
        this.endOfBatch = endOfBatch;
    }
    
    /**
     * Indicates if this query is part of a batch
     */
    public boolean isBatch() {
        return isBatch;
    }

    /**
     * Indicates if this query is the end of a batch
     */
    public boolean isEndOfBatch() {
        return isBatch() && endOfBatch;
    }

    /**
     * Returns the unique record ID
     */
    public String getRecordID() {
        return this.sID;
    }

    /**
     * Sets the query string
     * @param sQuery query string
     */
    public void setQueryString(String sQuery) {
        this.sQuery = sQuery;
    }

    /**
     * Returns the query type
     */
    public int getQueryType() {
        return iQueryType;
    }

    /**
     * Specifies whether the query should report events to the log
     * and the UI
     */
    public boolean getSilence() {
        return bSilence;
    }

    /**
     * Sets the query to silent. Silent means that no events are logged
     * @param silence  specifies if the query should run silently
     */
    public void setSilence(boolean silence) {
        this.bSilence = silence;
    }

    /**
     * Returns the requestors
     */
    public RequestorCollection getRequestors() {
        return requestors;
    }

    /**
     * Returns the query string. The query string is created if it doesn't 
     * exist as yet
     */
    public String getQueryString() {
        if (sQuery.equals("")) {
            sQuery = createQuery();
        }
        return sQuery;
    }

    /**
     * Creates a query string. Several flavours of column and values strings
     * are generated for the different flavours of query types.
     */
    private String createQuery() {
        boolean bCondition = false;
        if (vConditionValues != null && vConditionValues.size() > 0) {
            bCondition = true;

            String sValue = "";
            String sColumn = "";

            int counter = 0;
            for (Enumeration enumValues = vConditionValues.elements(); enumValues.hasMoreElements(); counter++) {
                Object oValue = enumValues.nextElement();
                sValue = oValue != null ? oValue.toString() : "";
                boolean integer = sValue.startsWith("'") ? false : true;
                
                String sValueLike = "";
                if (integer) {
                    sValueLike = sValue.toUpperCase();
                } else {
                    sValueLike = sValue.toUpperCase().substring(1, sValue.length() -1);
                }
                
                sColumn = conditionColumns[counter][1] + "." + conditionColumns[counter][0];

                if (StringUtils.isEmpty(sbValsDelete.toString())) {
                    sbValsDelete.append(sColumn + " = '" + sID + "'");
                }

                if (counter < vConditionValues.size() - 1) { // The end of the query has not been reached yet
                    sbVals.append(sValue + ", ");
                    sbColLikeVal.append(sUpper + "(" + sColumn + " )" + sLikeBegin + sValueLike + sLikeEnd + sAnd + '\n');
                    sbColLikeValOR.append(sUpper + "(" + sColumn + " )" + sLikeBegin + sValueLike  + sLikeEnd + sOr + '\n');
                    sbColIsVal.append(sUpper + "(" + sColumn + " ) = " + sValue.toUpperCase() + sAnd + '\n');
                    sbColIsValOR.append(sUpper + "(" + sColumn + " ) = " + sValue.toUpperCase() + sOr + '\n');
                    sbColIsValUpd.append(sColumn + " = " + sValue + ", " + '\n');
                } else { // Close the query
                    sbVals.append(sValue);
                    sbColLikeVal.append(sUpper + "(" + sColumn + " )" + sLikeBegin + sValueLike + sLikeEnd);
                    sbColLikeValOR.append(sUpper + "(" + sColumn + " )" + sLikeBegin + sValueLike + sLikeEnd);
                    sbColIsVal.append(sUpper + "(" + sColumn + " ) = " + sValue.toUpperCase());
                    sbColIsValOR.append(sUpper + "(" + sColumn + " ) = " + sValue.toUpperCase());
                    sbColIsValUpd.append(sColumn + " = " + sValue);
                }
            }
        }

        if (sOrderByField != null && !sOrderByField.trim().equals("")) {
            sbOrderBy.append(Query.sOrderBy + sOrderByField);
        }

        int counter = 0;
        for (Enumeration enum = vColumns.elements(); enum.hasMoreElements(); counter++) {
            String sColumn = (String) enum.nextElement();

            if (counter < vColumns.size() -1) {
                sbCols.append(sTableName + "." + sColumn + ", ");
            } else {
                sbCols.append(sTableName + "." + sColumn);
            }
        }


        String sVals = sbVals.toString();
        String sCols = sbCols.toString();
        String sColIsVal = sbColIsVal.toString();
        String sColIsValUpd = sbColIsValUpd.toString();
        String sColLikeVal = sbColLikeVal.toString();
        String sColIsValOr = sbColIsValOR.toString();
        String sColLikeValOr = sbColLikeValOR.toString();
        String sOrderBy = sbOrderBy.toString();

        switch (iQueryType) {

            case _DELETE :
                sQuery = sDelete + sTableName + '\n' + sWhere +
                         sbValsDelete + sClose;
                break;
                
            case _CREATE :
                sQuery = sCreate + sTableName + '\n' + "(" + sCols + ")" + sClose;
                break;

            case _SELECT :
                sTableName = sSelectTableDef;
                
                if (bCondition) {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + '\n'
                             + sWhere + sColLikeVal 
                             + sOrderBy + sClose;
                } else {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + sOrderBy + sClose;
                }
                break;

            case _SELECTPRECISE :
                sTableName = sSelectTableDef;
                if (bCondition) {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + '\n'
                             + sWhere + sColIsVal + sOrderBy + sClose;
                } else {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + sOrderBy + sClose;
                }
                break;

            case _INSERT :
                sQuery = sInsert + sTableName + " (" + sCols + ") " + '\n' +
                         sValues + "(" + sVals + ")" + sClose;
                break;

            case _UPDATE :
                sQuery = sUpdate + sTableName + '\n' + sSet + sColIsValUpd + '\n'
                        + sWhere + " ID " + "='" + sID + "'"
                        + sClose;
                break;

            case _SELECTPRECISEOR :
                if (bCondition) {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + '\n'
                             + sWhere + sColIsValOr + sOrderBy + sClose;
                } else {
                    sQuery = sSelect + sCols + '\n'
                             + sFrom + sTableName + sOrderBy + sClose;
                }
                break;

            case _SELECTOR :
                if (bCondition) {
                    sQuery = sSelect + sCols  + '\n'
                             + sFrom + sTableName + '\n'
                             + sWhere + sColLikeValOr + sOrderBy + sClose;
                } else {
                    sQuery = sSelect + sCols  + '\n'
                             + sFrom + sTableName + sOrderBy + sClose;
                }
                break;

            default :
        }
        return sQuery;
    }
}
