package net.sf.dc.http.covers;

import java.io.PrintWriter;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;

public class AmazonCoverSource {
    
    private final InternetSearchForm ui;
    private final AudioCD audioCD;
    
    private boolean keepOnRunning = true;
    
	private static final String SOAP_ACTION = "SearchArtistRequest";
	private static final String SOAP_URL = "http://soap.amazon.com/onca/soap3";
	private static final Pattern SOAP_PATTERN = Pattern.compile(".+?ProductName.+?>(.+?)</ProductName.+?Artist .+?>(.+?)</Artist.+?ImageUrlSmall.+?>(.+?)</ImageUrlSmall.+?ImageUrlMedium.+?>(.+?)</ImageUrlMedium.+?ImageUrlLarge.+?>(.+?)</ImageUrlLarge.*");

	public AmazonCoverSource(InternetSearchForm ui, AudioCD audioCD) {
        this.ui = ui;
        this.audioCD = audioCD;
    }
    
    public void stopSearch() {
        keepOnRunning = false; 
    }
    
	private String getSoapRequest(String artist, String album) {
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<SOAP-ENV:Envelope SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">");
		sb.append("<SOAP-ENV:Body>");
		sb.append("<ns1:ArtistSearchRequest xmlns:ns1=\"http://soap.amazon.com\" SOAP-ENC:root=\"1\">");
		sb.append("<v1>" + "<sort xsi:type=\"xsd:string\">+pmrank</sort>");
		sb.append("<artist xsi:type=\"xsd:string\">").append(artist).append("</artist>");
		sb.append("<devtag xsi:type=\"xsd:string\">D1R5WXCELBX4NF</devtag>");
		sb.append("<tag xsi:type=\"xsd:string\">accessonline</tag>");
		sb.append("<mode xsi:type=\"xsd:string\">music</mode>");
		sb.append("<keywords xsi:type=\"xsd:string\">").append(album).append("</keywords>");
		sb.append("<type xsi:type=\"xsd:string\">lite</type>");
		sb.append("<page xsi:type=\"xsd:int\">1</page>" + "</v1>");
		sb.append("</ns1:ArtistSearchRequest>" + "</SOAP-ENV:Body>");
		sb.append("</SOAP-ENV:Envelope>");
		
		return sb.toString();
	}
	
	private String sendSoapRequest(String request) throws Exception {
		//Set up the request
        URL url = new URL(SOAP_URL);
		HttpConnection connection = new HttpConnection(url);
		connection.setContentLength(request.length());
		connection.setToSoapRequest(SOAP_ACTION);

		//Send our xml request
        ui.addMessage("Sending search to Amazon.com");
		PrintWriter out = new PrintWriter(connection.getOutputStream());
		out.println(request);
		out.close();

		//Receive the answer
        String answer = connection.getString(url);
		return answer;
	}
	
	public void search() {
        String album = audioCD.getValue(AudioCD._A_TITLE).toString();
        String artist = audioCD.getValue(AudioCD._B_ARTIST).toString();
        
		//Transform the artist and album strings to better search
		artist = Utils.normalize(artist);
		album = Utils.normalize(album);

        String answer;
        try {
        	answer = sendSoapRequest(getSoapRequest(artist, album));
        	answer = answer.replaceAll("\n", "");
        	answer = answer.replaceAll("&amp;", "&");
        } catch (Exception exp) {
        	ui.addMessage("An error occured while trying to get data from Amazon.com:" + exp.getMessage());
            Messages.setMessage(exp);
            return;
        }
        
		int nextSubIndex = 0;
        Vector vURLs = new Vector(0);
		while (answer.length() > nextSubIndex && keepOnRunning) {
			Matcher matcher = SOAP_PATTERN.matcher(answer.substring(nextSubIndex));
			if (matcher.matches()) {
				//Retrieve the best url (non-empty image)
				String imageURL = "";
				for (int i = 5; i >= 3; i--) {
					imageURL = matcher.group(i);
					if (!imageURL.equals("")) {
                        vURLs.add(imageURL);
                        break;
                    }
				}
				nextSubIndex += matcher.end(matcher.groupCount());
			} else {
				break;
            }
		}
        retrieveImages(vURLs);
	}
    
    private void retrieveImages(Vector vURLs) {
        ui.addMessage("Start downloading " + vURLs.size() + " images");
        
        int results = vURLs.size();
        ui.initProgressBar(results);
        if (results == 0) {
        	ui.initProgressBar(1);
            ui.updateProgressBar(1);
            return;
        }
        
        int counter = 0;
        for (Enumeration enum = vURLs.elements(); enum.hasMoreElements(); counter++) {
            try {
                URL url = new URL(enum.nextElement().toString());
                HttpConnection connection = new HttpConnection(url);
                // check if this image is empty or not
                if (connection.getContentLength() != 807) {
                    // In case more then one image is found, add the record 
                    // again with the new image
                    if (counter > 0) {
                        AudioCD cd = new AudioCD();
                        Object[] values = audioCD.getValues();
                        
                        cd.setValues(values);
                        DCObject[]  children = audioCD.getChildren();
                        for (int i = 0; i < children.length; i++) {
                        	cd.addChild(children[i]);
                        }
                        
                        cd.setValue(AudioCD._E_PICTUREFRONT, connection.getImageAsBase64());                        
                        if (counter == 1) {
                        	ui.addMessage("Found more then one image! Adding new Audio CD items " +
                                          "for each extra image found");
                        }
                    	ui.addObject(cd);
                    } else {
                        audioCD.setValue(AudioCD._E_PICTUREFRONT, connection.getImageAsBase64());
                        ui.addMessage("Image retrieved and added");
                    }
                } else {
                    ui.addMessage("Skipping empty image");
                }
            } catch (Exception exp) {
                ui.addMessage("An error occured while retrieving cover:" + exp.getMessage());
                Messages.setMessage(exp);
            }
            ui.updateProgressBar(counter + 1);
        }
    }
}
